Package: polyCub
Title: Cubature over Polygonal Domains
Version: 0.9.4
Date: 2026-04-24
Authors@R: c(
    person("Sebastian", "Meyer",
           email = "seb.meyer@fau.de",
           role = c("aut","cre","trl"),
           comment = c(ORCID = "0000-0002-1791-9449")),
    person("Leonhard", "Held", role = "ths"),
    person("Michael", "Hoehle", role = "ths")
    )
Description: Numerical integration of continuously differentiable
    functions f(x,y) over simple closed polygonal domains.
    The following cubature methods are implemented:
    product Gauss cubature (Sommariva and Vianello, 2007,
    <doi:10.1007/s10543-007-0131-2>),
    the simple two-dimensional midpoint rule
    (wrapping 'spatstat.geom' functions), and
    adaptive cubature for radially symmetric functions via line
    integrate() along the polygon boundary (Meyer and Held, 2014,
    <doi:10.1214/14-AOAS743>, Supplement B).
    For simple integration along the axes, the 'cubature' package
    is more appropriate.
License: GPL-2
URL: https://github.com/bastistician/polyCub
BugReports: https://github.com/bastistician/polyCub/issues
Note: Building the package requires R >= 4.6.0 for \bibshow{} et al.
Depends: R (>= 3.4.0), methods
Imports: grDevices, graphics, stats, sp (>= 1.0-11)
Suggests: spatstat.geom, lattice, mvtnorm, statmod, sf, cubature,
        litedown (>= 0.9), microbenchmark
VignetteBuilder: litedown
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-04-24 09:20:44 UTC; smeyer
Author: Sebastian Meyer [aut, cre, trl] (ORCID:
    <https://orcid.org/0000-0002-1791-9449>),
  Leonhard Held [ths],
  Michael Hoehle [ths]
Maintainer: Sebastian Meyer <seb.meyer@fau.de>
Repository: CRAN
Date/Publication: 2026-04-24 10:00:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-04-24 13:22:07 UTC; unix
Archs: polyCub.so.dSYM

Package: polyCub
Title: Cubature over Polygonal Domains
Version: 0.9.4
Date: 2026-04-24
Authors@R: c(
    person("Sebastian", "Meyer",
           email = "seb.meyer@fau.de",
           role = c("aut","cre","trl"),
           comment = c(ORCID = "0000-0002-1791-9449")),
    person("Leonhard", "Held", role = "ths"),
    person("Michael", "Hoehle", role = "ths")
    )
Description: Numerical integration of continuously differentiable
    functions f(x,y) over simple closed polygonal domains.
    The following cubature methods are implemented:
    product Gauss cubature (Sommariva and Vianello, 2007,
    <doi:10.1007/s10543-007-0131-2>),
    the simple two-dimensional midpoint rule
    (wrapping 'spatstat.geom' functions), and
    adaptive cubature for radially symmetric functions via line
    integrate() along the polygon boundary (Meyer and Held, 2014,
    <doi:10.1214/14-AOAS743>, Supplement B).
    For simple integration along the axes, the 'cubature' package
    is more appropriate.
License: GPL-2
URL: https://github.com/bastistician/polyCub
BugReports: https://github.com/bastistician/polyCub/issues
Note: Building the package requires R >= 4.6.0 for \bibshow{} et al.
Depends: R (>= 3.4.0), methods
Imports: grDevices, graphics, stats, sp (>= 1.0-11)
Suggests: spatstat.geom, lattice, mvtnorm, statmod, sf, cubature,
        litedown (>= 0.9), microbenchmark
VignetteBuilder: litedown
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-04-24 09:20:44 UTC; smeyer
Author: Sebastian Meyer [aut, cre, trl] (ORCID:
    <https://orcid.org/0000-0002-1791-9449>),
  Leonhard Held [ths],
  Michael Hoehle [ths]
Maintainer: Sebastian Meyer <seb.meyer@fau.de>
Repository: CRAN
Date/Publication: 2026-04-24 10:00:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 15:39:58 UTC; unix
Archs: polyCub.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(xylist,Polygon)
S3method(xylist,Polygons)
S3method(xylist,SpatialPolygons)
S3method(xylist,default)
S3method(xylist,gpc.poly)
S3method(xylist,owin)
S3method(xylist,sfg)
export(.polyCub.iso)
export(as.owin.Polygon)
export(as.owin.Polygons)
export(as.owin.SpatialPolygons)
export(as.owin.gpc.poly)
export(checkintrfr)
export(circleCub.Gauss)
export(gpc2owin)
export(owin2gpc)
export(plotpolyf)
export(polyCub)
export(polyCub.SV)
export(polyCub.exact.Gauss)
export(polyCub.iso)
export(polyCub.midpoint)
export(sfg2gpc)
export(xylist)
exportMethods(coerce)
if(getRversion() >= "3.6.0") {  # delayed registration
    S3method(spatstat.geom::as.owin, SpatialPolygons)
    S3method(spatstat.geom::as.owin, Polygons)
    S3method(spatstat.geom::as.owin, Polygon)
}
if(getRversion() >= "3.6.0") {  # delayed registration
    S3method(spatstat.geom::as.owin, gpc.poly)
}
import(methods)
importClassesFrom(sp,Polygon)
importClassesFrom(sp,Polygons)
importClassesFrom(sp,SpatialPolygons)
importClassesFrom(sp,owin)
importFrom(grDevices,extendrange)
importFrom(grDevices,gray)
importFrom(grDevices,heat.colors)
importFrom(grDevices,xy.coords)
importFrom(graphics,image)
importFrom(graphics,lines)
importFrom(graphics,points)
importFrom(sp,coordinates)
importFrom(sp,plot)
importFrom(stats,cov2cor)
importFrom(stats,dist)
importFrom(stats,integrate)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importMethodsFrom(sp,plot)
useDynLib(polyCub, .registration = TRUE)

# polyCub <img src="man/figures/logo.png" align="right" alt="" width="120" />

The [R](https://www.R-project.org/) package **polyCub** implements
*cubature* (numerical integration) over *polygonal* domains.
It solves the problem of integrating a continuously differentiable
function f(x,y) over simple closed polygons.

For the special case of a rectangular domain along the axes, the
[**cubature**](https://CRAN.R-project.org/package=cubature)
package is more appropriate (cf.
[`CRAN Task View: Numerical Mathematics`](https://CRAN.R-project.org/view=NumericalMathematics)).


## Installation

You can install
[polyCub from CRAN](https://CRAN.R-project.org/package=polyCub)
via:

```R
install.packages("polyCub")
```

To install the development version from the
[GitHub repository](https://github.com/bastistician/polyCub), use:

```R
## install.packages("remotes")
remotes::install_github("bastistician/polyCub")
```


## Usage

The basic usage is:

```r
library("polyCub")
polyCub(polyregion, f)
```

* `polyregion` represents the integration domain as an object of class
`"owin"` (from **spatstat.geom**), `"gpc.poly"` (from **gpclib**),
`"SpatialPolygons"` (from **sp**), or `"(MULTI)POLYGON"` (from **sf**),
or even as a plain list of lists of vertex coordinates (`"xylist"`).

* `f` is the integrand and needs to take a two-column coordinate matrix
as its first argument.

The `polyCub()` function wraps the implemented cubature methods and
by default calls `polyCub.SV()`, a C-implementation of *product Gauss cubature*.
Directly calling the desired cubature function is preferable,
see the list below.

### Implemented cubature methods

1. `polyCub.SV()`:
   General-purpose **product Gauss cubature**
   (Sommariva and Vianello, 2007, *BIT Numerical Mathematics*,
   <https://doi.org/10.1007/s10543-007-0131-2>)

2. `polyCub.midpoint()`:
   Simple **two-dimensional midpoint rule** based on
   [**spatstat.geom**](https://CRAN.R-project.org/package=spatstat.geom)`::as.im.function()`

3. `polyCub.iso()`:
   Adaptive **cubature for radially symmetric functions**
   via line `integrate()` along the polygon boundary
   (Meyer and Held, 2014, *The Annals of Applied Statistics*,
   <https://doi.org/10.1214/14-AOAS743>, Supplement B, Section 2.4)

<!--
4. `polyCub.exact.Gauss()`:
   Accurate (but slow) **integration of the bivariate Gaussian density**
   based on polygon triangulation and
   [**mvtnorm**](https://CRAN.R-project.org/package=mvtnorm)`::pmvnorm()`
-->

For details and illustrations see the `vignette("polyCub")`
in the installed package or
[on CRAN](https://CRAN.R-project.org/package=polyCub/vignettes/polyCub.html).


## Applications

The **polyCub** package evolved from the need to integrate
so-called spatial interaction functions (Gaussian or power-law kernels)
over the observation region of a spatio-temporal point process.
Such epidemic models are implemented in
[**surveillance**](https://CRAN.R-project.org/package=surveillance).


## Feedback

Contributions are welcome!
Please submit suggestions or report bugs at
<https://github.com/bastistician/polyCub/issues>
or via e-mail to `maintainer("polyCub")`.


## License

The **polyCub** package is free and open source software, licensed under the GPLv2.

library("polyCub")

f <- function (s, sigma = 5)
{
    exp(-rowSums(s^2)/2/sigma^2) / (2*pi*sigma^2)
}

hexagon <- list(
    list(x = c(7.33, 7.33, 3, -1.33, -1.33, 3),
         y = c(-0.5, 4.5, 7, 4.5, -0.5, -3))
)

plotpolyf(hexagon, f, xlim = c(-8,8), ylim = c(-8,8))

par(mar = c(3,3,1,2))
polyCub.SV(hexagon, f, nGQ = 3, plot = TRUE)

nrow(polyCub.SV(hexagon, f = NULL, nGQ = 3)[[1]]$nodes)

polyCub.SVn <- function (polyregion, f, ..., nGQ = 20) {
    nw <- polyCub.SV(polyregion, f = NULL, ..., nGQ = nGQ)
    ## nw is a list with one element per polygon of 'polyregion'
    res <- sapply(nw, function (x)
        c(result = sum(x$weights * f(x$nodes, ...)), nEval = nrow(x$nodes)))
    structure(sum(res["result",]), nEval = sum(res["nEval",]))
}
polyCub.SVn(hexagon, f, nGQ = 3)

for (nGQ in c(1:5, 10, 20)) {
    result <- polyCub.SVn(hexagon, f, nGQ = nGQ)
    cat(sprintf("nGQ = %2i: %.12f (n=%i)\n", nGQ, result, attr(result, "nEval")))
}

library("spatstat.geom")
hexagon.owin <- owin(poly = hexagon)

par(mar = c(3,3,1,3), xaxs = "i", yaxs = "i")
polyCub.midpoint(hexagon.owin, f, eps = 0.5, plot = TRUE)

intrfr <- function (R, sigma = 5)
{
    (1 - exp(-R^2/2/sigma^2))/2/pi
}

polyCub.iso(hexagon, intrfr = intrfr, center = c(0,0))

bibentry(
    key = "meyer.held2014",
    bibtype = "Article",
    author = c(person("Sebastian", "Meyer"),
               person("Leonhard", "Held")),
    title = "Power-law models for infectious disease spread",
    journal = "Annals of Applied Statistics",
    year = "2014",
    volume = "8",
    number = "3",
    pages = "1612--1639",
    doi = "10.1214/14-AOAS743"
)

bibentry(
    key = "sommariva.vianello2007",
    bibtype = "Article",
    author = c(person(given = "Alvise", family = "Sommariva"),
               person(given = "Marco", family = "Vianello")),
    title = "Product {Gauss} cubature over polygons based on {Green}'s integration formula",
    journal = "BIT Numerical Mathematics",
    year = "2007",
    volume = "47",
    number = "2",
    pages = "441--453",
    doi = "10.1007/s10543-007-0131-2"
)

bibentry(
    key = "meyer2010",
    bibtype = "MastersThesis",
    author = "Sebastian Meyer",
    school = "Department of Statistics, LMU",
    title = "Spatio-Temporal Infectious Disease Epidemiology based on Point Processes",
    year = "2010",
    address = "Munich, Germany",
    url = "https://epub.ub.uni-muenchen.de/11703/"
)

bibentry(
    key = "R:spatstat.geom",
    bibtype = "Book",
    title = "Spatial Point Patterns: Methodology and Applications with {R}",
    author = "Adrian Baddeley and Ege Rubak and Rolf Turner",
    year = "2015",
    publisher = "Chapman and Hall/CRC Press",
    address = "London",
    isbn = "9781482210200"
)

################################################################################
### Internal Functions
###
### Copyright (C) 2009-2015,2017,2021,2026 Sebastian Meyer
###
### This file is part of the R package "polyCub",
### free software under the terms of the GNU General Public License, version 2,
### a copy of which is available at https://www.R-project.org/Licenses/.
################################################################################


## Check if a Polygon is Closed
## (first and last row of coordinate matrix are identical)
#' @importFrom grDevices xy.coords
isClosed <- function (coords)
{
    xycoords <- xy.coords(coords)[c("x","y")]
    n <- length(xycoords$x)
    return(identical(xycoords$x[1], xycoords$x[n]) &&
           identical(xycoords$y[1], xycoords$y[n]))
}


## Dot/Scalar Product of Two Vectors
dotprod <- function (x,y) sum(x*y)

## Euclidean Vector Norm (Length)
vecnorm <- function (x) sqrt(sum(x^2))

## Check if an R object is scalar (a numeric vector of length 1)
isScalar <- function (x) {
    length(x) == 1L && is.vector(x, mode = "numeric")
}


## Plot a Polygonal Domain (of Various Classes)
#' @importMethodsFrom sp plot
plot_polyregion <- function (polyregion, lwd=2, add=FALSE)
{
    if (is.vector(polyregion, mode="list")) { # internal xylist object
        stopifnot(add)
        lapply(polyregion, graphics::polygon, lwd=lwd)
        invisible()
    } else if (inherits(polyregion, "gpc.poly")) {
        plot(polyregion, poly.args=list(lwd=lwd), ann=FALSE, add=add)
    } else {
        if (inherits(polyregion, "Polygon"))
            polyregion <- sp::Polygons(list(polyregion), "ID")
        if (inherits(polyregion, "Polygons"))
            polyregion <- sp::SpatialPolygons(list(polyregion))
        ## plot call which works for "SpatialPolygons", "owin", and "sfg"
        plot(polyregion, lwd=lwd, axes=TRUE, main="", add=add)
    }
}

################################################################################
### Package Setup
###
### Copyright (C) 2009-2014,2018-2021,2023,2026 Sebastian Meyer
###
### This file is part of the R package "polyCub",
### free software under the terms of the GNU General Public License, version 2,
### a copy of which is available at https://www.R-project.org/Licenses/.
################################################################################


#' Cubature over Polygonal Domains
#'
#' The \R package \pkg{polyCub} implements
#' \emph{cubature} (numerical integration) over \emph{polygonal} domains.
#' It solves the problem of integrating a continuously differentiable
#' function \eqn{f(x,y)} over simple closed polygons.
#'
#' \pkg{polyCub} provides the following cubature methods:
#' \describe{
#' \item{\code{\link{polyCub.SV}}:}{
#' General-purpose \emph{product Gauss cubature}
#' \bibcitep{sommariva.vianello2007}
#' }
#' \item{\code{\link{polyCub.midpoint}}:}{
#' Simple \emph{two-dimensional midpoint rule} based on
#' \code{\link[spatstat.geom]{as.im.function}} from \CRANpkg{spatstat.geom}
#' \bibcitep{R:spatstat.geom}
#' }
#' \item{\code{\link{polyCub.iso}}:}{
#' Adaptive cubature for \emph{radially symmetric functions}
#' via line \code{\link{integrate}()} along the polygon boundary
#' \bibcitep{|meyer.held2014|Supplement B, Section 2.4}
#' }
#' }
#' A brief description and benchmark experiment of the above cubature
#' methods can be found in the \code{vignette("polyCub")}.
#'
#' There is also \code{\link{polyCub.exact.Gauss}}, intended to
#' accurately (but slowly) integrate the \emph{bivariate Gaussian density};
#' however, this implementation is disabled as of \pkg{polyCub} 0.9.0:
#' it needs a reliable implementation of polygon triangulation.
#'
#' \bibcitet{|meyer2010|Section 3.2}
#' discusses and compares some of these methods.
#'
#' @note To cite package \pkg{polyCub} in publications,
#' please use \code{citation("polyCub")}:
#'
#' \Sexpr[results=rd,stage=build]{tools::toRd(citation("polyCub"))}
#'
#' @author Sebastian Meyer
#' @references \bibshow{*}
#' @seealso
#' \code{vignette("polyCub")}
#'
#' For the special case of a rectangular domain along the axes
#' (e.g., a bounding box), the \CRANpkg{cubature} package is more appropriate.
"_PACKAGE"

################################################################################
### polyCub: Wrapper Function for the Various Cubature Methods
###
### Copyright (C) 2009-2013,2019 Sebastian Meyer
###
### This file is part of the R package "polyCub",
### free software under the terms of the GNU General Public License, version 2,
### a copy of which is available at https://www.R-project.org/Licenses/.
################################################################################


#' Wrapper Function for the Various Cubature Methods
#'
#' The wrapper function \code{polyCub} can be used to call specific cubature
#' methods via its \code{method} argument. It calls the \code{\link{polyCub.SV}}
#' function by default, which implements general-purpose product Gauss cubature.
#' The desired cubature function should usually be called directly.
#'
#' @inheritParams plotpolyf
#' @param f a two-dimensional real-valued function to be integrated over
#' \code{polyregion}. As its first argument it must take a coordinate matrix,
#' i.e., a numeric matrix with two columns, and it must return a numeric vector
#' of length the number of coordinates.\cr
#' For the \code{"exact.Gauss"} \code{method},
#' \code{f} is ignored since it is specific to the bivariate normal density.
#' @param method choose one of the implemented cubature methods (partial
#' argument matching is applied), see \code{help("\link{polyCub-package}")}
#' for an overview. Defaults to using product Gauss cubature
#' implemented in \code{\link{polyCub.SV}}.
#' @param ... arguments of \code{f} or of the specific \code{method}.
#' @param plot logical indicating if an illustrative plot of the numerical
#' integration should be produced.
#' @return The approximated integral of \code{f} over \code{polyregion}.
#' @seealso Details and examples in the \code{vignette("polyCub")}
#' and on the method-specific help pages.
#' @family polyCub-methods
#' @keywords math spatial
#' @export

polyCub <- function (polyregion, f,
                     method = c("SV", "midpoint", "iso", "exact.Gauss"), ...,
                     plot = FALSE)
{
    method <- match.arg(method)
    cl <- match.call()
    cl$method <- NULL
    cl[[1]] <- call("::", as.name("polyCub"),
                    as.name(paste("polyCub", method, sep=".")))
    if (method == "exact.Gauss") cl$f <- NULL
    int <- eval(cl, parent.frame())
    int
}

################################################################################
### as.owin.SpatialPolygons: Coerce "SpatialPolygons" to "owin"
###
### Copyright (C) 2012-2013,2015,2017-2018,2021 Sebastian Meyer
###
### This file is part of the R package "polyCub",
### free software under the terms of the GNU General Public License, version 2,
### a copy of which is available at https://www.R-project.org/Licenses/.
################################################################################


#' Coerce \code{"SpatialPolygons"} to \code{"owin"}
#'
#' Package \pkg{polyCub} implements \code{coerce}-methods
#' (\code{as(object, Class)}) to convert
#' \code{"\link[sp:SpatialPolygons-class]{SpatialPolygons}"}
#' (or \code{"\link[sp:Polygons-class]{Polygons}"}
#' or \code{"\link[sp:Polygon-class]{Polygon}"})
#' of package \CRANpkg{sp}
#' to \code{"\link[spatstat.geom:owin.object]{owin}"}
#' of package \CRANpkg{spatstat.geom}.
#' They are also available as \code{as.owin.*} functions to support
#' \code{\link{polyCub.midpoint}}.
#' @author Sebastian Meyer
#' @seealso \code{\link{xylist}}
#' @keywords spatial methods
#' @name coerce-sp-methods
#' @rdname coerce-sp-methods
#' @import methods
#' @importClassesFrom sp Polygon Polygons SpatialPolygons owin
#' @exportMethod coerce
#' @examples
#' if (require("spatstat.geom") && require("sp")) {
#'     diamond <- list(x = c(1,2,1,0), y = c(1,2,3,2))  # anti-clockwise
#'     diamond.owin <- owin(poly = diamond)
#'     diamond.sp <- Polygon(lapply(diamond, rev))      # clockwise
#'     stopifnot(identical(xylist(diamond.sp), list(diamond)))
#'     diamond.owin_from_sp <- as.owin(diamond.sp)
#'     stopifnot(all.equal(diamond.owin, diamond.owin_from_sp))
#'
#'     ## similarly works for Polygons and SpatialPolygons
#'     diamond.Ps <- as(diamond.sp, "Polygons")
#'     stopifnot(identical(diamond.owin, as.owin(diamond.Ps)))
#'     diamond.SpPs <- SpatialPolygons(list(diamond.Ps))
#'     stopifnot(identical(xylist(diamond.SpPs), list(diamond)))
#'     stopifnot(identical(diamond.owin, as.owin(diamond.SpPs)))
#' }
NULL

#' @param W an object of class \code{"SpatialPolygons"},
#' \code{"Polygons"}, or \code{"Polygon"}.
#' @param ... further arguments passed to \code{\link[spatstat.geom]{owin}}.
#' @rdname coerce-sp-methods
#' @export
#' @rawNamespace if(getRversion() >= "3.6.0") {  # delayed registration
#'     S3method(spatstat.geom::as.owin, SpatialPolygons)
#'     S3method(spatstat.geom::as.owin, Polygons)
#'     S3method(spatstat.geom::as.owin, Polygon)
#' }
as.owin.SpatialPolygons <- function (W, ...)
    spatstat.geom::owin(poly = xylist.SpatialPolygons(W), ...)

#' @rdname coerce-sp-methods
#' @export
as.owin.Polygons <- function (W, ...)
    spatstat.geom::owin(poly = xylist.Polygons(W), ...)

#' @rdname coerce-sp-methods
#' @export
as.owin.Polygon <- function (W, ...)
    spatstat.geom::owin(poly = xylist.Polygon(W), ...)


#' @name coerce,SpatialPolygons,owin-method
#' @rdname coerce-sp-methods
setAs(from = "SpatialPolygons", to = "owin",
      def = function (from) as.owin.SpatialPolygons(from))

#' @name coerce,Polygons,owin-method
#' @rdname coerce-sp-methods
setAs(from = "Polygons", to = "owin",
      def = function (from) as.owin.Polygons(from))

#' @name coerce,Polygon,owin-method
#' @rdname coerce-sp-methods
setAs(from = "Polygon", to = "owin",
      def = function (from) as.owin.Polygon(from))


#' @name coerce,Polygon,Polygons-method
#' @rdname coerce-sp-methods
setAs(from = "Polygon", to = "Polygons",
      def = function (from) Polygons(list(from), "Polygon"))

try(citation("polyCub"), silent = TRUE) # OK for R CMD build/INSTALL/check
## NOTE: Requires installation, so is incompatible with building directly
##       from source via `R CMD Rd2pdf pkgdir` or `tools::pkg2HTML(dir=)`.
##       A more general approach would be:
##   pkgdir <- tools:::Rd_macros_package_dir()
##   if (file.exists(descfile <- file.path(pkgdir, "DESCRIPTION")) && # not for base
##       read.dcf(descfile, "Package") == "polyCub") {
##       readCitationFile(file.path(pkgdir, "inst", "CITATION"))
##   } else { # foreign use of 'polyCub::CITATION': access installed package
##       citation("polyCub")
##   }

bibentry(
    key = "meyer.held2014",
    bibtype = "Article",
    author = c(person(given = "Sebastian", family = "Meyer"),
               person(given = "Leonhard", family = "Held")),
    title = "Power-law models for infectious disease spread",
    journal = "Annals of Applied Statistics",
    year = "2014",
    volume = "8",
    number = "3",
    pages = "1612--1639",
    doi = "10.1214/14-AOAS743"
)

bibentry(
    key = "sommariva.vianello2007",
    bibtype = "Article",
    author = c(person(given = "Alvise", family = "Sommariva"),
               person(given = "Marco", family = "Vianello")),
    title = "Product {Gauss} cubature over polygons based on {Green}'s integration formula",
    journal = "BIT Numerical Mathematics",
    year = "2007",
    volume = "47",
    number = "2",
    pages = "441--453",
    doi = "10.1007/s10543-007-0131-2"
)

bibentry(
    key = "meyer2010",
    bibtype = "MastersThesis",
    author = person(given = "Sebastian", family = "Meyer"),
    school = "Department of Statistics, LMU",
    title = "Spatio-Temporal Infectious Disease Epidemiology based on Point Processes",
    year = "2010",
    address = "Munich, Germany",
    url = "https://epub.ub.uni-muenchen.de/11703/"
)

################################################################################
### polyCub.iso: Cubature of Isotropic Functions over Polygonal Domains
###
### Copyright (C) 2013-2018 Sebastian Meyer
###
### This file is part of the R package "polyCub",
### free software under the terms of the GNU General Public License, version 2,
### a copy of which is available at https://www.R-project.org/Licenses/.
################################################################################


#' Cubature of Isotropic Functions over Polygonal Domains
#'
#' \code{polyCub.iso} numerically integrates a radially symmetric function
#' \eqn{f(x,y) = f_r(||(x,y)-\boldsymbol{\mu}||)}{f(x,y) = f_r(||(x,y)-\mu||)},
#' with \eqn{\mu} being the center of isotropy, over a polygonal domain.
#' It internally approximates a line integral along the polygon boundary using
#' \code{\link{integrate}}. The integrand requires the antiderivative of
#' \eqn{r f_r(r)}), which should be supplied as argument \code{intrfr}
#' (\code{f} itself is only required if \code{check.intrfr=TRUE}).
#' The two-dimensional integration problem thereby reduces to an efficient
#' adaptive quadrature in one dimension.
#' If \code{intrfr} is not available analytically, \code{polyCub.iso} can use a
#' numerical approximation (meaning \code{integrate} within \code{integrate}),
#' but the general-purpose cubature method \code{\link{polyCub.SV}} might be
#' more efficient in this case.
#' See Meyer and Held (2014, Supplement B, Section 2.4) for mathematical
#' details.
#'
#' @inheritParams plotpolyf
#' @param intrfr a \code{function(R, ...)}, which implements the (analytical)
#' antiderivative of \eqn{r f_r(r)} from 0 to \code{R}. The first argument
#' must be vectorized but not necessarily named \code{R}.\cr
#' If \code{intrfr} is missing, it will be approximated numerically via
#' \code{\link{integrate}(function(r, ...)
#' r * f(cbind(x0 + r, y0), ...), 0, R, ..., control=control)},
#' where \code{c(x0, y0)} is the \code{center} of isotropy.
#' Note that \code{f} will \emph{not} be checked for isotropy.
#' @param ... further arguments for \code{f} or \code{intrfr}.
#' @param center numeric vector of length 2, the center of isotropy.
#' @param control list of arguments passed to \code{\link{integrate}}, the
#' quadrature rule used for the line integral along the polygon boundary.
#' @param check.intrfr logical (or numeric vector) indicating if
#' (for which \code{r}'s) the supplied \code{intrfr} function should be
#' checked against a numeric approximation. This check requires \code{f}
#' to be specified. If \code{TRUE}, the set of test
#' \code{r}'s defaults to a \code{\link{seq}} of length 20 from 1 to
#' the maximum absolute x or y coordinate of any edge of the \code{polyregion}.
#' @param plot logical indicating if an image of the function should be plotted
#' together with the polygonal domain, i.e.,
#' \code{\link{plotpolyf}(polyregion, f, \dots)}.
#' @return The approximate integral of the isotropic function
#' \code{f} over \code{polyregion}.\cr
#' If the \code{intrfr} function is provided (which is assumed to be exact), an
#' upper bound for the absolute integration error is appended to the result as
#' attribute \code{"abs.error"}. It equals the sum of the absolute errors
#' reported by all \code{\link{integrate}} calls
#' (there is one for each edge of \code{polyregion}).
#' @author Sebastian Meyer
#'
#' The basic mathematical formulation of this efficient integration for radially
#' symmetric functions was ascertained with great support by
#' Emil Hedevang (2013), Dept. of Mathematics, Aarhus University, Denmark.
#' @references
#' Hedevang, E. (2013). Personal communication at the Summer School on Topics in
#' Space-Time Modeling and Inference (May 2013, Aalborg, Denmark).
#'
#' Meyer, S. and Held, L. (2014).
#' Power-law models for infectious disease spread.
#' \emph{The Annals of Applied Statistics}, \bold{8} (3), 1612-1639.
#' \doi{10.1214/14-AOAS743}
#' @seealso
#' \code{system.file("include", "polyCubAPI.h", package = "polyCub")}
#' for a full C-implementation of this cubature method (for a \emph{single}
#' polygon). The corresponding C-routine \code{polyCub_iso} can be used by
#' other \R packages, notably \CRANpkg{surveillance}, via \samp{LinkingTo: polyCub}
#' (in the \file{DESCRIPTION}) and \samp{#include <polyCubAPI.h>} (in suitable
#' \file{/src} files). Note that the \code{intrfr} function must then also be
#' supplied as a C-routine. An example can be found in the package tests.
#' @keywords math spatial
#' @family polyCub-methods
#' @example examples/polyCub.iso.R
#' @importFrom stats integrate
#' @export

polyCub.iso <- function (polyregion, f, intrfr, ..., center,
                         control = list(), check.intrfr = FALSE, plot = FALSE)
{
    polys <- xylist(polyregion) # transform to something like "owin$bdry"
                                # which means anticlockwise vertex order with
                                # first vertex not repeated
    getError <- !missing(intrfr) # can't estimate error of double approximation
    center <- as.vector(center, mode = "numeric")
    stopifnot(length(center) == 2L, is.finite(center))

    ## check 'intrfr' function
    rs <- if (isTRUE(check.intrfr)) {
        seq(1, max(abs(unlist(lapply(polys, "[", c("x","y"))))), length.out=20L)
    } else if (identical(check.intrfr, FALSE)) {
        numeric(0L)
    } else {
        check.intrfr
    }
    intrfr <- checkintrfr(intrfr, f, ..., center=center, control=control, rs=rs)

    ## plot polygon and function image
    if (plot) plotpolyf(polys, f, ...)

    ## do the cubature over all polygons of the 'polys' list
    .polyCub.iso(polys, intrfr, ..., center=center,
                 control=control, .witherror=getError)
}


#' Check the Integral of \eqn{r f_r(r)}
#'
#' This function is auxiliary to \code{\link{polyCub.iso}}.
#' The (analytical) integral of \eqn{r f_r(r)} from 0 to \eqn{R} is checked
#' against a numeric approximation using \code{\link{integrate}} for various
#' values of the upper bound \eqn{R}. A warning is issued if inconsistencies
#' are found.
#'
#' @inheritParams polyCub.iso
#' @param rs numeric vector of upper bounds for which to check the validity of
#' \code{intrfr}. If it has length 0 (default), no checks are performed.
#' @param tolerance of \code{\link{all.equal.numeric}} when comparing
#' \code{intrfr} results with numerical integration. Defaults to the
#' relative tolerance used for \code{integrate}.
#' @return The \code{intrfr} function. If it was not supplied, its quadrature
#' version using \code{integrate} is returned.
#' @importFrom stats integrate
#' @export
checkintrfr <- function (intrfr, f, ..., center, control = list(),
                         rs = numeric(0L), tolerance = control$rel.tol)
{
    doCheck <- length(rs) > 0L
    if (!missing(f)) {
        f <- match.fun(f)
        rfr <- function (r, ...)
            r * f(cbind(center[1L]+r, center[2L], deparse.level=0L), ...)
        quadrfr1 <- function (R, ...) integrate(rfr, 0, R, ...)$value
        if (length(control))
            body(quadrfr1)[[2L]] <- as.call(c(as.list(body(quadrfr1)[[2L]]),
                                             control))
        quadrfr <- function (R, ...)
            vapply(X = R, FUN = quadrfr1, FUN.VALUE = 0, ..., USE.NAMES = FALSE)
        if (missing(intrfr)) {
            return(quadrfr)
        } else if (doCheck) {
            cat("Checking 'intrfr' against a numeric approximation ... ")
            stopifnot(is.vector(rs, mode="numeric"))
            if (is.null(tolerance))
                tolerance <- eval(formals(integrate)$rel.tol)
            ana <- intrfr(rs, ...)
            num <- quadrfr(rs, ...)
            if (!isTRUE(comp <- all.equal(num, ana, tolerance=tolerance))) {
                cat("\n->", comp, "\n")
                warning("'intrfr' might be incorrect: ", comp)
            } else cat("OK\n")
        }
    } else if (doCheck) {
        stop("numerical verification of 'intrfr' requires 'f'")
    }

    match.fun(intrfr)
}


#' @description
#' \code{.polyCub.iso} is a \dQuote{bare-bone} version of \code{polyCub.iso}.
#' @rdname polyCub.iso
#' @param polys something like \code{owin$bdry}, but see \code{\link{xylist}}.
#' @param .witherror logical indicating if an upper bound for the absolute
#' integration error should be attached as an attribute to the result?
#' @export
.polyCub.iso <- function (polys, intrfr, ..., center,
                          control = list(), .witherror = FALSE)
{
    ints <- lapply(polys, polyCub1.iso,
                   intrfr, ..., center=center,
                   control=control, .witherror=.witherror)
    if (.witherror) {
        res <- sum(vapply(X=ints, FUN="[", FUN.VALUE=0, 1L, USE.NAMES=FALSE))
        attr(res, "abs.error") <-
            sum(vapply(X=ints, FUN="[", FUN.VALUE=0, 2L, USE.NAMES=FALSE))
        res
    } else {
        sum(unlist(ints, recursive=FALSE, use.names=FALSE))
    }
}

## cubature method for a single polygon
polyCub1.iso <- function (poly, intrfr, ..., center,
                          control = list(), .witherror = TRUE)
{
    xy <- cbind(poly[["x"]], poly[["y"]], deparse.level=0L)
    nedges <- nrow(xy)
    intedges <- erredges <- numeric(nedges)
    for (i in seq_len(nedges)) {
        v0 <- xy[i, ] - center
        v1 <- xy[if (i==nedges) 1L else i+1L, ] - center
        int <- lineInt(v0, v1, intrfr, ..., control=control)
        intedges[i] <- int$value
        erredges[i] <- int$abs.error
    }
    int <- sum(intedges)
    ## if (!is.null(poly$hole) && !isTRUE(all.equal(0, int))) {
    ##     if ((1 - 2 * as.numeric(poly$hole)) * sign(int) == -1)
    ##         warning("wrong sign if positive integral")
    ## }
    if (.witherror) {
        c(int, sum(erredges))
    } else {
        int
    }
}

## line integral for one edge
#' @importFrom stats integrate
lineInt <- function (v0, v1, intrfr, ..., control = list())
{
    d <- v1 - v0
    num <- v1[2L]*v0[1L] - v1[1L]*v0[2L]  # = d[2]*p[,1] - d[1]*p[,2]
                                          # for any point p on the edge
    if (num == 0) { # i.e., if 'center' is part of this polygon edge
        return(list(value = 0, abs.error = 0))
    }
    integrand <- function (t) {
        ## get the points on the edge corresponding to t
        p <- cbind(v0[1L] + t*d[1L], v0[2L] + t*d[2L], deparse.level=0L)
        norm2 <- .rowSums(p^2, length(t), 2L)
        ints <- intrfr(sqrt(norm2), ...)
        ##ints[is.infinite(ints)] <- 1e300
        num * ints / norm2
    }
    if (length(control)) {              # use slower do.call()-construct
        do.call("integrate", c(list(integrand, 0, 1), control))
    } else {
        integrate(integrand, 0, 1)
    }
}

if (!requireNamespace("spatstat.geom"))
    q("no")
library("polyCub")

## bivariate, isotropic Gaussian density
f <- function (s, mean, sd)
    dnorm(s[,1], mean=mean[1], sd=sd) * dnorm(s[,2], mean=mean[2], sd=sd)

## circular domain represented by a polygon
r <- 5
center <- c(3,2)
npoly <- 128
disc.owin <- spatstat.geom::disc(radius=r, centre=center, npoly=npoly)

## parameters for f
m <- c(1,1)
sd <- 3

## target value of the integral over the _polygonal_ circle
intExact <- 0.65844436  # taken from exact.Gauss cubature (below)
stopIfDiff <- function(int, ...)
    if(!isTRUE(all.equal.numeric(intExact, int, ..., check.attributes = FALSE))) {
        if (is.call(cl <- substitute(int))) cl <- cl[1]
        stop(deparse(cl), " result not equal to reference value")
    }

## reproduce saved reference value
if (identical(Sys.getenv("R_GPCLIBPERMIT"), "true") &&
    local({pkg <- "gpclib"; requireNamespace(pkg)}) && # undeclared ...
    requireNamespace("mvtnorm"))
    stopIfDiff(polyCub.exact.Gauss(disc.owin, mean=m, Sigma=sd^2*diag(2)),
               tolerance = 1e-8)

## exact value of the integral over the _real_ circle
stopIfDiff(circleCub.Gauss(center=center, r=r, mean=m, sd=sd),
           tolerance = 0.001)  # agreement depends on 'npoly'

## polyCub.midpoint
stopIfDiff(polyCub.midpoint(disc.owin, f, mean=m, sd=sd, dimyx=500),
           tolerance = 0.001)

## polyCub.SV
intC <- polyCub.SV(disc.owin, f, mean=m, sd=sd, nGQ=3, engine="C")
intR <- polyCub.SV(disc.owin, f, mean=m, sd=sd, nGQ=3, engine="R")
stopifnot(all.equal(intC, intR))
stopIfDiff(intC, tolerance = 0.0001)

## polyCub.iso (using a numerical approximation of intrfr)
stopIfDiff(polyCub.iso(disc.owin, f, mean=m, sd=sd, center=m))

library("polyCub")

## function to call an R CMD with environment variables
## 'env' specified as a named character vector
Rcmd <- function (args, env = character(), ...) {
    stopifnot(is.vector(env, mode = "character"),
              !is.null(names(env)))
    if (.Platform$OS.type == "windows") {
        if (length(env)) {
            ## the 'env' argument of system2() is not supported on Windows
            setenv <- function (envs) {
                old <- Sys.getenv(names(envs), unset = NA, names = TRUE)
                set <- !is.na(envs)
                if (any(set)) do.call(Sys.setenv, as.list(envs[set]))
                if (any(!set)) Sys.unsetenv(names(envs)[!set])
                invisible(old)
            }
            oldenv <- setenv(env)
            on.exit(setenv(oldenv))
        }
        system2(command = file.path(R.home("bin"), "Rcmd.exe"),
                args = args, ...)
    } else {
        system2(command = file.path(R.home("bin"), "R"),
                args = c("CMD", args),
                env = paste(names(env), env, sep = "="), ...)
    }
}

## test compilation with #include <polyCubAPI.h> for the polyCub_iso C-routine
message("compiling polyiso_powerlaw.c using R CMD SHLIB")
shlib_error <- Rcmd(
    args = c("SHLIB", "--clean", "polyiso_powerlaw.c"),
    env = c("PKG_CPPFLAGS" = paste0(
                "-I", system.file("include", package="polyCub")
            ),
            "R_TESTS" = "")
)
if (shlib_error) {
    warning("failed to build the shared object/DLL for the polyCub_iso example")
    q("no")
}

## load shared object/DLL
myDLL <- paste0("polyiso_powerlaw", .Platform$dynlib.ext)
loadNamespace("polyCub")
dyn.load(myDLL)

## R function calling C_polyiso_powerlaw
polyiso_powerlaw <- function (xypoly, logpars, center,
                              subdivisions = 100L,
                              rel.tol = .Machine$double.eps^0.25,
                              abs.tol = rel.tol, stop.on.error = TRUE)
{
    .C("C_polyiso_powerlaw",
       as.double(xypoly$x), as.double(xypoly$y), as.integer(length(xypoly$x)),
       as.double(logpars),
       as.double(center[1L]), as.double(center[2L]),
       as.integer(subdivisions), as.double(abs.tol), as.double(rel.tol),
       as.integer(stop.on.error),
       value = double(1L), abserr = double(1L), neval = integer(1L)
       )[c("value", "abserr", "neval")]
}


## example polygon and function parameters
diamond <- list(x = c(1,2,1,0), y = c(1,2,3,2))
logpars <- log(c(0.5, 1))
center <- c(0.5,2.5)  # lies on an edge (to cover that case as well)

(res <- polyiso_powerlaw(xypoly = diamond,
                         logpars = logpars,
                         center = center))


## compare with R implementation
intrfr.powerlaw <- function (R, logpars)
{
    sigma <- exp(logpars[[1L]])
    d <- exp(logpars[[2L]])
    if (d == 1) {
        R - sigma * log(R/sigma + 1)
    } else if (d == 2) {
        log(R/sigma + 1) - R/(R+sigma)
    } else {
        (R*(R+sigma)^(1-d) - ((R+sigma)^(2-d) - sigma^(2-d))/(2-d)) / (1-d)
    }
}
(orig <- polyCub:::polyCub1.iso(poly = diamond,
                                intrfr = intrfr.powerlaw,
                                logpars = logpars,
                                center = center))

stopifnot(all.equal(res$value, orig[1L]))
stopifnot(all.equal(res$abserr, orig[2L]))

## microbenchmark::microbenchmark(
##     polyCub:::polyCub1.iso(diamond, intrfr.powerlaw, logpars, center=center),
##     polyiso_powerlaw(diamond, logpars, center=center),
##     times = 1000)
## ## 150 mus vs. 20 mus

dyn.unload(myDLL)
file.remove(myDLL)

################################################################################
### xylist: Convert Various Polygon Classes to a Simple List of Vertices
###
### Copyright (C) 2012-2014,2017,2021 Sebastian Meyer
###
### This file is part of the R package "polyCub",
### free software under the terms of the GNU General Public License, version 2,
### a copy of which is available at https://www.R-project.org/Licenses/.
################################################################################


#' Convert Various Polygon Classes to a Simple List of Vertices
#'
#' Different packages concerned with spatial data use different polygon
#' specifications, which sometimes becomes very confusing (see Details below).
#' To be compatible with the various polygon classes, package \pkg{polyCub}
#' uses an S3 class \code{"xylist"}, which represents a polygonal domain
#' (of potentially multiple polygons) by its core feature only: a list of lists
#' of vertex coordinates (see the "Value" section below).
#' The generic function \code{xylist} can deal with the
#' following polygon classes:
#' \itemize{
#' \item \code{"\link[spatstat.geom:owin.object]{owin}"} from package \pkg{spatstat.geom}
#' \item \code{"gpc.poly"} from package \pkg{gpclib}
#' \item \code{"\link[sp:Polygons-class]{Polygons}"} from package \pkg{sp}
#' (as well as \code{"\link[sp:Polygon-class]{Polygon}"} and
#' \code{"\link[sp:SpatialPolygons-class]{SpatialPolygons}"})
#' \item \code{"\link[sf:st_polygon]{(MULTI)POLYGON}"} from package \pkg{sf}
#' }
#' The (somehow useless) default \code{xylist}-method
#' does not perform any transformation but only ensures that the polygons are
#' not closed (first vertex not repeated).
#'
#' Polygon specifications differ with respect to:
#' \itemize{
#' \item is the first vertex repeated?
#' \item which ring direction represents holes?
#' }
#' Package overview:
#' \describe{
#' \item{\pkg{spatstat.geom}:}{\code{"owin"} does \emph{not repeat} the
#' first vertex, and anticlockwise = normal boundary, clockwise = hole.
#' This convention is also used for the return value of \code{xylist}.}
#' \item{\pkg{sp}:}{\emph{Repeat} first vertex at the end (closed),
#' anticlockwise = hole, clockwise = normal boundary}
#' \item{\pkg{sf}:}{\emph{Repeat} first vertex at the end (closed),
#' clockwise = hole, anticlockwise = normal boundary;
#' \emph{however}, \pkg{sf} does not check the ring direction by default, so
#' it cannot be relied upon.}
#' \item{\pkg{gpclib}:}{There seem to be no such conventions
#' for polygons of class \code{"gpc.poly"}.}
#' }
#' Thus, for polygons from \pkg{sf} and \pkg{gpclib}, \code{xylist} needs
#' to check the ring direction, which makes these two formats the least
#' efficient for integration domains in \pkg{polyCub}.
#'
#' @param object an object of one of the supported spatial classes.
#' @param ... (unused) argument of the generic.
#' @return Applying \code{xylist} to a polygon object, one gets a simple list,
#' where each component (polygon) is a list of \code{"x"} and \code{"y"}
#' coordinates. These represent vertex coordinates following \pkg{spatstat.geom}'s
#' \code{"owin"} convention (anticlockwise order for exterior boundaries,
#' without repeating any vertex).
#' @author Sebastian Meyer
#' @name xylist
#' @keywords spatial methods
#' @export
xylist <- function (object, ...) UseMethod("xylist")

#' @rdname xylist
#' @export
xylist.owin <- function (object, ...)
{
    spatstat.geom::as.polygonal(object)$bdry
}

#' @rdname xylist
#' @export
xylist.sfg <- function (object, ...)
{
    assert_polygonal_sfg(object)
    obj <- sf::st_sfc(object, check_ring_dir = TRUE)[[1L]]
    ## it would be more efficient to use sf's check_ring_dir() directly
    ## unfortunately, that function is not exported from sf (0.9-7)
    if (inherits(obj, "MULTIPOLYGON"))
        obj <- unlist(obj, recursive = FALSE)
    lapply(obj, function (coords) {
        idx <- seq_len(nrow(coords) - 1L)
        list(x = coords[idx, 1L], y = coords[idx, 2L])
    })
}

#' @rdname xylist
#' @export
xylist.gpc.poly <- function (object, ...)
{
    xylist.owin(gpc2owin(object, check = FALSE))
}

#' @rdname xylist
#' @export
xylist.SpatialPolygons <- function (object, reverse = TRUE, ...)
{
    unlist(lapply(object@polygons, xylist.Polygons, reverse=reverse, ...),
           recursive=FALSE, use.names=FALSE)
}

#' @rdname xylist
#' @param reverse logical (\code{TRUE}) indicating if the vertex order of the
#' \pkg{sp} classes should be reversed to get the \code{xylist}/\code{owin}
#' convention.
#' @importFrom sp coordinates
#' @export
xylist.Polygons <- function (object, reverse = TRUE, ...)
{
    lapply(object@Polygons, function (sr) {
        coords <- coordinates(sr)
        n <- nrow(coords) - 1L   # number of vertices
        idxs <- if (reverse) seq.int(n,1) else seq_len(n)
        list(x = coords[idxs,1L], y = coords[idxs,2L])
             #area = sr@area, hole = sr@hole
    })
}

#' @rdname xylist
#' @import methods
#' @export
xylist.Polygon <- function (object, reverse = TRUE, ...)
    xylist.Polygons(as(object,"Polygons"), reverse=reverse, ...)

#' @rdname xylist
#' @importFrom grDevices xy.coords
#' @export
xylist.default <- function (object, ...) {
    lapply(object, function (xy) {
        poly <- xy.coords(xy)[c("x","y")]
        if (isClosed(poly)) {
            sel <- seq_len(length(poly$x) - 1L)
            poly$x <- poly$x[sel]
            poly$y <- poly$y[sel]
        }
        poly
    })
}

################################################################################
### polyCub.midpoint: Two-Dimensional Midpoint Rule
###
### Copyright (C) 2009-2015,2017,2021 Sebastian Meyer
###
### This file is part of the R package "polyCub",
### free software under the terms of the GNU General Public License, version 2,
### a copy of which is available at https://www.R-project.org/Licenses/.
################################################################################


#' Two-Dimensional Midpoint Rule
#'
#' The surface is converted to a binary pixel image
#' using the \code{\link[spatstat.geom]{as.im.function}} method from package
#' \CRANpkg{spatstat.geom} (Baddeley et al., 2015).
#' The integral under the surface is then approximated as the
#' sum over (pixel area * f(pixel midpoint)).
#'
#' @inheritParams plotpolyf
#' @param polyregion a polygonal integration domain.
#' It can be any object coercible to the \pkg{spatstat.geom} class
#' \code{"\link[spatstat.geom]{owin}"} via a corresponding
#' \code{\link[spatstat.geom]{as.owin}}-method.
#' Note that this includes polygons of the classes \code{"gpc.poly"} and
#' \code{"\link[sp:SpatialPolygons-class]{SpatialPolygons}"}, because
#' \pkg{polyCub} defines methods \code{\link{as.owin.gpc.poly}} and
#' \code{\link{as.owin.SpatialPolygons}}, respectively.
#' \pkg{sf} also registers suitable \code{as.owin} methods for its
#' \code{"\link[sf:st_polygon]{(MULTI)POLYGON}"} classes.
#' @param eps width and height of the pixels (squares),
#' see \code{\link[spatstat.geom]{as.mask}}.
#' @param dimyx number of subdivisions in each dimension,
#' see \code{\link[spatstat.geom]{as.mask}}.
#' @param plot logical indicating if an illustrative plot of the numerical
#' integration should be produced.
#' @return The approximated value of the integral of \code{f} over
#' \code{polyregion}.
#' @references
#' Baddeley A, Rubak E, Turner R (2015).
#' \emph{Spatial Point Patterns: Methodology and Applications with R}.
#' Chapman and Hall/CRC Press, London.
#' @keywords math spatial
#' @family polyCub-methods
#' @importFrom sp plot
#' @importFrom grDevices gray
#' @example examples/setting.R
#' @example examples/polyCub.midpoint.R
#' @export

polyCub.midpoint <- function (polyregion, f, ...,
                              eps = NULL, dimyx = NULL, plot = FALSE)
{
    ## as.im needs separate x and y arguments
    fxy <- function (x, y, ...) f(cbind(x,y), ...)

    ## calculate pixel values of fxy
    IM <- tryCatch(
        spatstat.geom::as.im.function(X = fxy, W = polyregion, ...,
                                      eps = eps, dimyx = dimyx),
        error = function (e) {
        ## if eps was to small such that the dimensions of the image would
        ## be too big then the operation matrix(TRUE, nr, nc) throws an
        ## error. (try e.g. devnull <- matrix(TRUE, 1e6,1e6))
        ## unfortunately, it is not clear what we should do in this case ...
        stop("inapplicable choice of bandwidth (eps=", format(eps),
             ") in midpoint rule:\n", e)
    })

### ILLUSTRATION ###
    if (plot) {
        spatstat.geom::plot.im(IM, axes=TRUE, col=gray(31:4/35), main="")
        ## add evaluation points
        #with(IM, points(expand.grid(xcol, yrow), col=!is.na(v), cex=0.5))
        plot(polyregion, add=TRUE, poly.args=list(lwd=2), lwd=2)
        ##<- two 'lwd'-specifications such that it works with owin and gpc.poly
    }
####################

    ## return the approximated integral
    spatstat.geom::integral.im(IM)
}

################################################################################
### Integration of the Isotropic Gaussian Density over Circular Domains
###
### Copyright (C) 2013-2014 Sebastian Meyer
###
### This file is part of the R package "polyCub",
### free software under the terms of the GNU General Public License, version 2,
### a copy of which is available at https://www.R-project.org/Licenses/.
################################################################################


#' Integration of the Isotropic Gaussian Density over Circular Domains
#'
#' This function calculates the integral of the bivariate, isotropic Gaussian
#' density (i.e., \eqn{\Sigma} = \code{sd^2*diag(2)}) over a circular domain
#' via the cumulative distribution function \code{pchisq} of the (non-central)
#' Chi-Squared distribution (Abramowitz and Stegun, 1972, Formula 26.3.24).
#'
#' @references
#' Abramowitz, M. and Stegun, I. A. (1972).
#' Handbook of Mathematical Functions with Formulas, Graphs, and Mathematical
#' Tables. New York: Dover Publications.
#' @param center numeric vector of length 2 (center of the circle).
#' @param r numeric (radius of the circle). Several radii may be supplied.
#' @param mean numeric vector of length 2
#'             (mean of the bivariate Gaussian density).
#' @param sd numeric (common standard deviation of the isotropic
#'           Gaussian density in both dimensions).
#' @return The integral value (one for each supplied radius).
#' @note The non-centrality parameter of the evaluated chi-squared distribution
#' equals the squared distance between the \code{mean} and the
#' \code{center}. If this becomes too large, the result becomes inaccurate, see
#' \code{\link{pchisq}}.
#' @keywords math spatial
#' @importFrom stats pchisq
#' @export
#' @examples
#' circleCub.Gauss(center=c(1,2), r=3, mean=c(4,5), sd=6)
#'
#' ## compare with cubature over a polygonal approximation of a circle
#' \dontrun{## (this example requires gpclib)
#' disc.poly <- spatstat.geom::disc(radius=3, centre=c(1,2), npoly=32)
#' polyCub.exact.Gauss(disc.poly, mean=c(4,5), Sigma=6^2*diag(2))
#' }

circleCub.Gauss <- function (center, r, mean, sd)
{
    stopifnot(isScalar(sd), length(center) == 2, length(mean) == 2)
    pchisq((r/sd)^2, df=2, ncp=sum(((center-mean)/sd)^2))
}

################################################################################
### polyCub.SV: Product Gauss Cubature over Polygonal Domains
###
### Copyright (C) 2009-2014,2017-2018 Sebastian Meyer
###
### This file is part of the R package "polyCub",
### free software under the terms of the GNU General Public License, version 2,
### a copy of which is available at https://www.R-project.org/Licenses/.
################################################################################


#' Product Gauss Cubature over Polygonal Domains
#'
#' Product Gauss cubature over polygons as proposed by
#' Sommariva and Vianello (2007).
#'
#' @inheritParams plotpolyf
#' @param f a two-dimensional real-valued function to be integrated over
#' \code{polyregion} (or \code{NULL} to only compute nodes and weights).
#' As its first argument it must take a coordinate matrix, i.e., a
#' numeric matrix with two columns, and it must return a numeric vector of
#' length the number of coordinates.
#' @param nGQ degree of the one-dimensional Gauss-Legendre quadrature rule
#' (default: 20) as implemented in function \code{\link[statmod]{gauss.quad}}
#' of package \CRANpkg{statmod}. Nodes and weights up to \code{nGQ=60} are cached
#' in \pkg{polyCub}, for larger degrees \pkg{statmod} is required.
#' @param alpha base-line of the (rotated) polygon at \eqn{x = \alpha} (see
#' Sommariva and Vianello (2007) for an explication). If \code{NULL} (default),
#' the midpoint of the x-range of each polygon is chosen if no \code{rotation}
#' is performed, and otherwise the \eqn{x}-coordinate of the rotated point
#' \code{"P"} (see \code{rotation}). If \code{f} has its maximum value at the
#' origin \eqn{(0,0)}, e.g., the bivariate Gaussian density with zero mean,
#' \code{alpha = 0} is a reasonable choice.
#' @param rotation logical (default: \code{FALSE}) or a list of points
#' \code{"P"} and \code{"Q"} describing the preferred direction. If
#' \code{TRUE}, the polygon is rotated according to the vertices \code{"P"} and
#' \code{"Q"}, which are farthest apart (see Sommariva and Vianello, 2007). For
#' convex polygons, this rotation guarantees that all nodes fall inside the
#' polygon.
#' @param engine character string specifying the implementation to use.
#' Up to \pkg{polyCub} version 0.4-3, the two-dimensional nodes and weights
#' were computed by \R functions and these are still available by setting
#' \code{engine = "R"}.
#' The new C-implementation is now the default (\code{engine = "C"}) and
#' requires approximately 30\% less computation time.\cr
#' The special setting \code{engine = "C+reduce"} will discard redundant nodes
#' at (0,0) with zero weight resulting from edges on the base-line
#' \eqn{x = \alpha} or orthogonal to it.
#' This extra cleaning is only worth its cost for computationally intensive
#' functions \code{f} over polygons which really have some edges on the
#' baseline or parallel to the x-axis.  Note that the old \R
#' implementation does not have such unset zero nodes and weights.
#' @param plot logical indicating if an illustrative plot of the numerical
#' integration should be produced.
#' @return The approximated value of the integral of \code{f} over
#' \code{polyregion}.\cr
#' In the case \code{f = NULL}, only the computed nodes and weights are
#' returned in a list of length the number of polygons of \code{polyregion},
#' where each component is a list with \code{nodes} (a numeric matrix with
#' two columns), \code{weights} (a numeric vector of length
#' \code{nrow(nodes)}), the rotation \code{angle}, and \code{alpha}.
#' @author Sebastian Meyer\cr
#' These R and C implementations of product Gauss cubature are based on the
#' original \acronym{MATLAB} implementation \code{polygauss} by Sommariva and
#' Vianello (2007), which is available under the GNU GPL (>=2) license from
#' \url{https://www.math.unipd.it/~alvise/software.html}.
#' @references
#' Sommariva, A. and Vianello, M. (2007):
#' Product Gauss cubature over polygons based on Green's integration formula.
#' \emph{BIT Numerical Mathematics}, \bold{47} (2), 441-453.
#' \doi{10.1007/s10543-007-0131-2}
#' @keywords math spatial
#' @family polyCub-methods
#' @importFrom graphics points
#' @example examples/setting.R
#' @example examples/polyCub.SV.R
#' @export

polyCub.SV <- function (polyregion, f, ...,
                        nGQ = 20, alpha = NULL, rotation = FALSE, engine = "C",
                        plot = FALSE)
{
    polys <- xylist(polyregion) # transform to something like "owin$bdry"
                                # which means anticlockwise vertex order with
                                # first vertex not repeated
    stopifnot(isScalar(nGQ), nGQ > 0,
              is.null(alpha) || (isScalar(alpha) && !is.na(alpha)))

    ## COMPUTE NODES AND WEIGHTS OF 1D GAUSS QUADRATURE RULE.
    ## DEGREE "N" (as requested) (ORDER GAUSS PRIMITIVE)
    nw_N <- gauss.quad(nGQ)
    ## DEGREE "M" = N+1 (ORDER GAUSS INTEGRATION)
    nw_M <- gauss.quad(nGQ + 1)

    ## Special case f=NULL: compute and return nodes and weights only
    if (is.null(f)) {
        return(lapply(X = polys, FUN = polygauss, nw_MN = c(nw_M, nw_N),
                      alpha = alpha, rotation = rotation, engine = engine))
    }

    ## Cubature over every single polygon of the "polys" list
    f <- match.fun(f)
    int1 <- function (poly) {
        nw <- polygauss(poly, c(nw_M, nw_N), alpha, rotation, engine)
        fvals <- f(nw$nodes, ...)
        cubature_val <- sum(nw$weights * fvals)
        ## if (!isTRUE(all.equal(0, cubature_val))) {
        ## if ((1 - 2 * as.numeric(poly$hole)) * sign(cubature_val) == -1)
        ## warning("wrong sign if positive integral")
        ## }
        cubature_val
    }
    respolys <- vapply(X=polys, FUN=int1, FUN.VALUE=0, USE.NAMES=FALSE)
    int <- sum(respolys)

### ILLUSTRATION ###
    if (plot) {
        plotpolyf(polys, f, ..., use.lattice=FALSE)
        for (i in seq_along(polys)) {
            nw <- polygauss(polys[[i]], c(nw_M, nw_N), alpha, rotation, engine)
            points(nw$nodes, cex=0.6, pch = i) #, col=1+(nw$weights<=0)
        }
    }
###################

    int
}

## this wrapper provides a partially memoized version of
## unname(statmod::gauss.quad(n, kind="legendre"))
gauss.quad <- function (n)
{
    if (n <= 61) { # results cached in R/sysdata.rda
        .NWGL[[n]]
    } else if (requireNamespace("statmod")) {
        unname(statmod::gauss.quad(n = n, kind = "legendre"))
    } else {
        stop("package ", sQuote("statmod"), " is required for nGQ > 60")
    }
}


#' Calculate 2D Nodes and Weights of the Product Gauss Cubature
#'
#' @param xy list with elements \code{"x"} and \code{"y"} containing the
#' polygon vertices in \emph{anticlockwise} order (otherwise the result of the
#' cubature will have a negative sign) with first vertex not repeated at the
#' end (like \code{owin.object$bdry}).
#' @param nw_MN unnamed list of nodes and weights of one-dimensional Gauss
#' quadrature rules of degrees \eqn{N} and \eqn{M=N+1} (as returned by
#' \code{\link[statmod]{gauss.quad}}): \code{list(s_M, w_M, s_N, w_N)}.
#' @inherit polyCub.SV params references
#' @keywords internal
#' @useDynLib polyCub, .registration = TRUE

polygauss <- function (xy, nw_MN, alpha = NULL, rotation = FALSE, engine = "C")
{
    ## POLYGON ROTATION

    xyrot <- if (identical(FALSE, rotation)) {
        if (is.null(alpha)) { # choose midpoint of x-range
            xrange <- range(xy[["x"]])
            alpha <- (xrange[1L] + xrange[2L]) / 2
        }
        angle <- 0
        xy[c("x", "y")]
    } else {
        ## convert to coordinate matrix
        xy <- cbind(xy[["x"]], xy[["y"]], deparse.level=0)
        ## determine P and Q
        if (identical(TRUE, rotation)) { # automatic choice of rotation angle
            ## such that for a convex polygon all nodes fall inside the polygon
            QP <- vertexpairmaxdist(xy)
            Q <- QP[1L,,drop=TRUE]
            P <- QP[2L,,drop=TRUE]
        } else if (is.list(rotation)) {  # predefined rotation
            P <- rotation$P
            Q <- rotation$Q
            stopifnot(is.vector(P, mode="numeric") && length(P) == 2L,
                      is.vector(Q, mode="numeric") && length(Q) == 2L)
            stopifnot(any(P != Q))
            rotation <- TRUE
        } else {
            stop("'rotation' must be logical or a list of points ",
                 "\"P\" and \"Q\"")
        }
        rotmat <- rotmatPQ(P,Q)
        angle <- attr(rotmat, "angle")
        if (is.null(alpha)) {
            Prot <- rotmat %*% P
            alpha <- Prot[1]
        }
        xyrot <- xy %*% t(rotmat)   # = t(rotmat %*% t(xy))
        ## convert back to list
        list(x = xyrot[,1L,drop=TRUE], y = xyrot[,2L,drop=TRUE])
    }

    ## number of vertices
    L <- length(xyrot[[1L]])


    ## COMPUTE 2D NODES AND WEIGHTS.

    if (engine == "R") {

        toIdx <- c(seq.int(2, L), 1L)
        nwlist <- mapply(.polygauss.side,
                         xyrot[[1L]], xyrot[[2L]],
                         xyrot[[1L]][toIdx], xyrot[[2L]][toIdx],
                         MoreArgs = c(nw_MN, alpha),
                         SIMPLIFY = FALSE, USE.NAMES = FALSE)

        nodes <- c(lapply(nwlist, "[[", 1L),
                   lapply(nwlist, "[[", 2L),
                   recursive=TRUE)
        dim(nodes) <- c(length(nodes)/2, 2L)
        weights <- unlist(lapply(nwlist, "[[", 3L),
                          recursive=FALSE, use.names=FALSE)

    } else { # use C-implementation

        ## degrees of cubature and vector template for results
        M <- length(nw_MN[[1L]])
        N <- length(nw_MN[[3L]])
        zerovec <- double(L*M*N)

        ## rock'n'roll
        nwlist <- .C(C_polygauss,
                     as.double(xyrot[[1L]]), as.double(xyrot[[2L]]),
                     as.double(nw_MN[[1L]]), as.double(nw_MN[[2L]]),
                     as.double(nw_MN[[3L]]), as.double(nw_MN[[4L]]),
                     as.double(alpha),
                     as.integer(L), as.integer(M), as.integer(N),
                     x = zerovec, y = zerovec, w = zerovec)[c("x", "y", "w")]

        nodes <- cbind(nwlist[[1L]], nwlist[[2L]], deparse.level=0)
        weights <- nwlist[[3L]]

        ## remove unset nodes from edges on baseline or orthogonal to it
        ## (note that the R implementation does not return such redundant nodes)
        if (engine == "C+reduce" && any(unset <- weights == 0)) {
            nodes <- nodes[!unset,]
            weights <- weights[!unset]
        }

    }

    ## back-transform rotated nodes by t(t(rotmat) %*% t(nodes))
    ## (inverse of rotation matrix is its transpose)
    list(nodes = if (rotation) nodes %*% rotmat else nodes,
         weights = weights, angle = angle, alpha = alpha)
}


## The working horse .polygauss.side below is an R translation
## of the original MATLAB implementation by Sommariva and Vianello (2007).

.polygauss.side <- function (x1, y1, x2, y2, s_loc, w_loc, s_N, w_N, alpha)
{
    if ((x1 == alpha && x2 == alpha) || (y2 == y1))
        ## side lies on base-line or is orthogonal to it -> skip
        return(NULL)

    if (x2 == x1) { # side is parallel to base-line => degree N
        s_loc <- s_N
        w_loc <- w_N
    }

    half_pt_x <- (x1+x2)/2
    half_length_x <- (x2-x1)/2

    half_pt_y <- (y1+y2)/2
    half_length_y <- (y2-y1)/2

    ## GAUSSIAN POINTS ON THE SIDE.
    x_gauss_side <- half_pt_x + half_length_x * s_loc
    y_gauss_side <- half_pt_y + half_length_y * s_loc

    scaling_fact_minus <- (x_gauss_side - alpha) / 2

    ## construct nodes and weights: x and y coordinates ARE STORED IN MATRICES.
    ## A COUPLE WITH THE SAME INDEX IS A POINT, i.e. P_i=(x(k),y(k)).
    ## Return in an unnamed list of nodes_x, nodes_y, weights
    ## (there is no need for c(nodes_x) and c(weights))
    list(
        alpha + tcrossprod(scaling_fact_minus, s_N + 1), # degree_loc x N
        rep.int(y_gauss_side, length(s_N)),              # length: degree_loc*N
        tcrossprod(half_length_y*scaling_fact_minus*w_loc, w_N) # degree_loc x N
    )
}

## NOTE: The above .polygauss.side() function is already efficient R code.
##       Passing via C only at this deep level (see below) turned out to be
##       slower than staying with R! However, stepping into C already for
##       looping over the edges in polygauss() improves the speed.
## ## @useDynLib polyCub C_polygauss_side
## .polygauss.side <- function (x1, y1, x2, y2, s_M, w_M, s_N, w_N, alpha)
## {
##     if ((x1 == alpha && x2 == alpha) || (y2 == y1))
##         ## side lies on base-line or is orthogonal to it -> skip
##         return(NULL)
##
##     parallel2baseline <- x2 == x1  # side is parallel to base-line => degree N
##     M <- length(s_M)
##     N <- length(s_N)
##     loc <- if (parallel2baseline) N else M
##     zerovec <- double(loc * N)
##     .C(C_polygauss_side,
##        as.double(x1), as.double(y1), as.double(x2), as.double(y2),
##        as.double(if (parallel2baseline) s_N else s_M),
##        as.double(if (parallel2baseline) w_N else w_M),
##        as.double(s_N), as.double(w_N), as.double(alpha),
##        as.integer(loc), as.integer(N),
##        x = zerovec, y = zerovec, w = zerovec)[c("x", "y", "w")]
## }


#' @importFrom stats dist
vertexpairmaxdist <- function (xy)
{
    ## compute euclidean distance matrix
    distances <- dist(xy)
    size <- attr(distances, "Size")

    ## select two points with maximum distance
    maxdistidx <- which.max(distances)
    lowertri <- seq_along(distances) == maxdistidx
    mat <- matrix(FALSE, size, size)
    mat[lower.tri(mat)] <- lowertri
    QPidx <- which(mat, arr.ind=TRUE, useNames=FALSE)[1L,]
    xy[QPidx,]
}

rotmatPQ <- function (P, Q)
{
    direction_axis <- (Q-P) / vecnorm(Q-P)

    ## determine rotation angle [radian]
    rot_angle_x <- acos(direction_axis[1L])
    rot_angle_y <- acos(direction_axis[2L])
    rot_angle <- if (rot_angle_y <= pi/2) {
        if (rot_angle_x <= pi/2) -rot_angle_y else rot_angle_y
    } else {
        if (rot_angle_x <= pi/2) pi-rot_angle_y else rot_angle_y
    }

    ## rotation matrix
    rot_matrix <- diag(cos(rot_angle), nrow=2L)
    rot_matrix[2:3] <- c(-1,1) * sin(rot_angle) # clockwise rotation
    structure(rot_matrix, angle=rot_angle)
}

library("polyCub")

### Regression tests

hexagon <- list(
    list(x = c(7.33, 7.33, 3, -1.33, -1.33, 3),
         y = c(-0.5, 4.5, 7, 4.5, -0.5, -3))
)
f <- function (s) (rowSums(s^2)+1)^-2
##plotpolyf(hexagon, f)

## isotropic cubature can handle control list for integrate()
## previously, passing control arguments did not work
int1 <- polyCub.iso(hexagon, f, center=c(0,0), control=list(rel.tol=1e-3))
int2 <- polyCub.iso(hexagon, f, center=c(0,0), control=list(rel.tol=1e-8))
## results are almost but not identical
stopifnot(all.equal(int1, int2, tolerance = 1e-3),
          !identical(int1, int2))

################################################################################
### Conversion between polygonal "owin" and "gpc.poly"
###
### Copyright (C) 2012-2015,2017-2018,2021 Sebastian Meyer
###
### This file is part of the R package "polyCub",
### free software under the terms of the GNU General Public License, version 2,
### a copy of which is available at https://www.R-project.org/Licenses/.
################################################################################


#' Conversion between polygonal \code{"owin"} and \code{"gpc.poly"}
#'
#' Package \pkg{polyCub} implements converters between the classes
#' \code{"\link[spatstat.geom:owin.object]{owin}"} of package \CRANpkg{spatstat.geom}
#' and \code{"gpc.poly"} of package \CRANpkg{gpclib}.
#'
#' @param object an object of class \code{"gpc.poly"} or \code{"owin"},
#' respectively.
#' @return The converted polygon of class \code{"gpc.poly"} or \code{"owin"},
#' respectively. If package \pkg{gpclib} is not available,
#' \code{owin2gpc} will just return the \code{pts} slot of the
#' \code{"gpc.poly"} (no formal class) with a warning.
#' @author Sebastian Meyer
#' @note The converter \code{owin2gpc} requires the package
#' \pkg{gpclib} for the formal class definition of a \code{"gpc.poly"}.
#' It will produce vertices ordered according to the \pkg{sp} convention,
#' i.e. clockwise for normal boundaries and anticlockwise for holes, where,
#' however, the first vertex is \emph{not} repeated!
#' @seealso \code{\link{xylist}}
#' @name coerce-gpc-methods
#' @rdname coerce-gpc-methods
#' @keywords spatial methods
#' @import methods
#' @export
#' @examplesIf requireNamespace("spatstat.geom")
#' ## use example polygons from
#' example(plotpolyf, ask = FALSE)
#' letterR  # a simple "xylist"
#'
#' letterR.owin <- spatstat.geom::owin(poly = letterR)
#' letterR.gpc_from_owin <- owin2gpc(letterR.owin)
#' ## warns if "gpclib" is unavailable
#'
#' if (is(letterR.gpc_from_owin, "gpc.poly")) {
#'   letterR.xylist_from_gpc <- xylist(letterR.gpc_from_owin)
#'   stopifnot(all.equal(letterR, lapply(letterR.xylist_from_gpc, `[`, 1:2)))
#'   letterR.owin_from_gpc <- gpc2owin(letterR.gpc_from_owin)
#'   stopifnot(all.equal(letterR.owin, letterR.owin_from_gpc))
#' }
owin2gpc <- function (object)
{
    object <- spatstat.geom::as.polygonal(object)

    ## determine hole flags of the individual polygons
    hole <- spatstat.geom::summary.owin(object)$areas < 0

    ## reverse vertices and set hole flags
    pts <- mapply(
        FUN = function (poly, hole) {
            list(x = rev.default(poly$x), y = rev.default(poly$y),
                 hole = hole)
        },
        poly = object$bdry, hole = hole,
        SIMPLIFY = FALSE, USE.NAMES = FALSE)

    ## formal class
    if (know_gpc.poly()) {
        new("gpc.poly", pts = pts)
    } else {
        warning("formal class \"gpc.poly\" not available")
        pts
    }
}

#' @rdname coerce-gpc-methods
#' @param ... further arguments passed to \code{\link[spatstat.geom]{owin}}.
#' @export
gpc2owin <- function (object, ...)
{
    ## first convert to an "owin" without checking areas etc.
    ## to determine the hole status according to vertex order (area)
    res <- spatstat.geom::owin(poly = object@pts, check = FALSE)
    holes_owin <- spatstat.geom::summary.owin(res)$areas < 0
    ## or directly lapply spatstat.utils::is.hole.xypolygon

    ## now fix the vertex order
    bdry <- mapply(
        FUN = function (poly, owinhole) {
            if (poly$hole != owinhole) {
                poly$x <- rev(poly$x)
                poly$y <- rev(poly$y)
            }
            poly
        },
        poly = object@pts, owinhole = holes_owin,
        SIMPLIFY = FALSE, USE.NAMES = FALSE)

    ## now really convert to owin with appropriate vertex order
    spatstat.geom::owin(poly = bdry, ...)
}

#' @rdname coerce-gpc-methods
#' @param W an object of class \code{"gpc.poly"}.
#' @export
#' @rawNamespace if(getRversion() >= "3.6.0") {  # delayed registration
#'     S3method(spatstat.geom::as.owin, gpc.poly)
#' }
as.owin.gpc.poly <- function (W, ...)
{
    gpc2owin(W, ...)
}


## check for the formal class "gpc.poly"
#' @import methods
know_gpc.poly <- function ()
{
    isClass("gpc.poly") #|| requireNamespace("gpclib", quietly = TRUE)
}

################################################################################
### plotpolyf: Plot Polygonal Domain on Image of Bivariate Function
###
### Copyright (C) 2013-2014,2018,2021 Sebastian Meyer
###
### This file is part of the R package "polyCub",
### free software under the terms of the GNU General Public License, version 2,
### a copy of which is available at https://www.R-project.org/Licenses/.
################################################################################


#' Plot Polygonal Domain on Image of Bivariate Function
#'
#' Produces a combined plot of a polygonal domain and an image of a bivariate
#' function, using either \code{\link[lattice:levelplot]{lattice::levelplot}}
#' or \code{\link{image}}.
#'
#' @param polyregion a polygonal domain.
#' The following classes are supported:
#' \code{"\link[spatstat.geom]{owin}"} from package \pkg{spatstat.geom},
#' \code{"gpc.poly"} from \pkg{gpclib},
#' \code{"\link[sp:SpatialPolygons-class]{SpatialPolygons}"},
#' \code{"\link[sp:Polygons-class]{Polygons}"},
#' and \code{"\link[sp:Polygon-class]{Polygon}"} from package \pkg{sp}, as well as
#' \code{"\link[sf:st_polygon]{(MULTI)POLYGON}"} from package \pkg{sf}.
#' (For these classes, \pkg{polyCub} knows how to get an \code{\link{xylist}}.)
#' @param f a two-dimensional real-valued function.
#' As its first argument it must take a coordinate matrix, i.e., a
#' numeric matrix with two columns, and it must return a numeric vector of
#' length the number of coordinates.
#' @param ... further arguments for \code{f}.
#' @param npixel numeric vector of length 1 or 2 setting the number of pixels
#' in each dimension.
#' @param cuts number of cut points in the \eqn{z} dimension.
#' The range of function values will be divided into \code{cuts+1} levels.
#' @param col color vector used for the function levels.
#' @param lwd line width of the polygon edges.
#' @param xlim,ylim numeric vectors of length 2 setting the axis limits.
#' \code{NULL} means using the bounding box of \code{polyregion}.
#' @param use.lattice logical indicating if \pkg{lattice} graphics
#' (\code{\link[lattice]{levelplot}}) should be used.
#' @param print.args a list of arguments passed to \code{\link[lattice]{print.trellis}}
#' for plotting the produced \code{\link[lattice:trellis.object]{"trellis"}} object
#' (given \code{use.lattice = TRUE}). The latter will be returned without
#' explicit \code{print}ing if \code{print.args} is not a list.
#' @author Sebastian Meyer
#' @keywords hplot
#' @example examples/plotpolyf.R
#' @importFrom grDevices extendrange heat.colors
#' @importFrom graphics image
#' @export

plotpolyf <- function (polyregion, f, ..., npixel = 100, cuts = 15,
                       col = rev(heat.colors(cuts+1)), lwd = 3,
                       xlim = NULL, ylim = NULL,
                       use.lattice = TRUE, print.args = list())
{
    polys <- if (inherits(polyregion, "gpc.poly")) { # avoid xylist.gpc.poly
                 polyregion@pts  # ring direction is irrelevant for plotting
             } else xylist(polyregion)
    npixel <- rep(npixel, length.out = 2L)

    ## make two-dimensional grid
    if (is.null(xlim))
        xlim <- extendrange(unlist(lapply(polys, "[[", "x"), use.names=FALSE))
    if (is.null(ylim))
        ylim <- extendrange(unlist(lapply(polys, "[[", "y"), use.names=FALSE))
    xgrid <- seq(xlim[1L], xlim[2L], length.out = npixel[1L])
    ygrid <- seq(ylim[1L], ylim[2L], length.out = npixel[2L])
    xygrid <- expand.grid(x=xgrid, y=ygrid, KEEP.OUT.ATTRS=FALSE)

    ## compute function values on the grid
    xygrid$fval <- f(as.matrix(xygrid, rownames.force = FALSE), ...)

    ## plot
    if (use.lattice && requireNamespace("lattice")) {
        mypanel <- function(...) {
            lattice::panel.levelplot(...)
            lapply(polys, function(xy) lattice::panel.polygon(xy, lwd=lwd))
        }
        trobj <- lattice::levelplot(fval ~ x*y, data=xygrid, aspect="iso",
                                    cuts=cuts, col.regions=col, panel=mypanel)
        if (is.list(print.args)) {
            do.call("print", c(alist(x=trobj), print.args))
        } else trobj
    } else {
        image(xgrid, ygrid, matrix(xygrid$fval, npixel[1L], npixel[2L]),
              col=col, xlab="x", ylab="y", asp=1)
        plot_polyregion(polyregion, lwd=lwd, add=TRUE)
    }
}

################################################################################
### Convert polygonal "sfg" to "gpc.poly" (for polyCub.exact.Gauss)
###
### Copyright (C) 2021 Sebastian Meyer
###
### This file is part of the R package "polyCub",
### free software under the terms of the GNU General Public License, version 2,
### a copy of which is available at https://www.R-project.org/Licenses/.
################################################################################


#' Convert polygonal \code{"sfg"} to \code{"gpc.poly"}
#'
#' Package \pkg{polyCub} implements a converter from class
#' \code{"\link[sf:st_polygon]{(MULTI)POLYGON}"} of package \CRANpkg{sf}
#' to \code{"gpc.poly"} of package \CRANpkg{gpclib}
#' such that \code{\link{polyCub.exact.Gauss}}
#' can be used with simple feature polygons.
#'
#' @param object a \code{"POLYGON"} or \code{"MULTIPOLYGON"} \code{"sfg"} object.
#' @return The converted polygon of class \code{"gpc.poly"}.
#' If package \pkg{gpclib} is not available,
#' \code{sfg2gpc} will just return the \code{pts} slot of the
#' \code{"gpc.poly"} (no formal class) with a warning.
#' @author Sebastian Meyer
#' @note Package \pkg{gpclib} is required for the formal class
#' definition of a \code{"gpc.poly"}.
#' @seealso \code{\link{xylist}}
#' @keywords spatial methods
#' @import methods
#' @export
#' @examplesIf requireNamespace("sf")
#' ## use example polygons from
#' example(plotpolyf, ask = FALSE)
#' letterR  # a simple "xylist"
#'
#' letterR.sfg <- sf::st_polygon(lapply(letterR, function(xy)
#'     rbind(cbind(xy$x, xy$y), c(xy$x[1], xy$y[1]))))
#' letterR.sfg
#' stopifnot(identical(letterR, xylist(letterR.sfg)))
#' \dontshow{
#' stopifnot(identical(rep(letterR, 2),
#'     xylist(sf::st_multipolygon(list(letterR.sfg, letterR.sfg)))))
#' }
#' ## convert sf "POLYGON" to a "gpc.poly"
#' letterR.gpc_from_sfg <- sfg2gpc(letterR.sfg)
#' letterR.gpc_from_sfg
#' \dontshow{
#' if (is(letterR.gpc_from_sfg, "gpc.poly") && requireNamespace("spatstat.geom")) {
#'     letterR.xylist_from_gpc <- xylist(letterR.gpc_from_sfg) # with hole info
#'     stopifnot(identical(letterR, lapply(letterR.xylist_from_gpc, `[`, 1:2)))
#' }}
sfg2gpc <- function (object)
{
    assert_polygonal_sfg(object)

    ## determine hole flags of the individual polygons
    if (inherits(object, "MULTIPOLYGON")) {
        hole <- unlist(lapply(object, seq_along)) > 1L
        object <- unlist(object, recursive = FALSE)
    } else {
        hole <- seq_along(object) > 1L
    }

    pts <- mapply(
        FUN = function (coords, hole) {
            idx <- seq_len(nrow(coords) - 1L)
            list(x = coords[idx, 1L], y = coords[idx, 2L], hole = hole)
        },
        coords = object, hole = hole,
        SIMPLIFY = FALSE, USE.NAMES = FALSE)

    if (know_gpc.poly()) {
        new("gpc.poly", pts = pts)
    } else {
        warning("formal class \"gpc.poly\" not available")
        pts
    }
}

assert_polygonal_sfg <- function (object)
{
    if (!inherits(object, c("POLYGON", "MULTIPOLYGON")))
        stop("only *polygonal* SF geometries are supported")
    invisible(object)
}

################################################################################
### polyCub.exact.Gauss: Quasi-Exact Cubature of the Bivariate Normal Density
###
### Copyright (C) 2009-2018,2021-2023 Sebastian Meyer
###
### This file is part of the R package "polyCub",
### free software under the terms of the GNU General Public License, version 2,
### a copy of which is available at https://www.R-project.org/Licenses/.
################################################################################


#' Quasi-Exact Cubature of the Bivariate Normal Density (DEFUNCT)
#'
#' This cubature method is \strong{defunct} as of \pkg{polyCub} version 0.9.0.
#' It relied on \code{tristrip()} from package \CRANpkg{gpclib} for polygon
#' triangulation, but that package did not have a \acronym{FOSS} license and
#' was no longer maintained on a mainstream repository.\cr
#' Contributions to resurrect this cubature method are welcome: an alternative
#' implementation for constrained polygon triangulation is needed, see
#' \url{https://github.com/bastistician/polyCub/issues/2}.
#'
#' The bivariate Gaussian density can be integrated based on a triangulation of
#' the (transformed) polygonal domain, using formulae from the
#' Abramowitz and Stegun (1972) handbook (Section 26.9, Example 9, pp. 956f.).
#' This method is quite cumbersome because the A&S formula is only for triangles
#' where one vertex is the origin (0,0). For each triangle
#' we have to check in which of the 6 outer
#' regions of the triangle the origin (0,0) lies and adapt the signs in the
#' formula appropriately: \eqn{(AOB+BOC-AOC)} or \eqn{(AOB-AOC-BOC)} or
#' \eqn{(AOB+AOC-BOC)} or \eqn{(AOC+BOC-AOB)} or \ldots.
#' However, the most time consuming step is the
#' evaluation of \code{\link[mvtnorm]{pmvnorm}}.
#'
#' @param polyregion a \code{"gpc.poly"} polygon or
#' something that can be coerced to this class, e.g., an \code{"owin"} polygon
#' (via \code{\link{owin2gpc}}), or an \code{"sfg"} polygon (via
#' \code{\link{sfg2gpc}}).
#' @param mean,Sigma mean and covariance matrix of the bivariate normal density
#' to be integrated.
#' @param plot logical indicating if an illustrative plot of the numerical
#' integration should be produced. Note that the \code{polyregion} will be
#' transformed (shifted and scaled).
#' @return The integral of the bivariate normal density over \code{polyregion}.
#' Two attributes are appended to the integral value:
#' \item{nEval}{
#' number of triangles over which the standard bivariate normal density had to
#' be integrated, i.e. number of calls to \code{\link[mvtnorm]{pmvnorm}} and
#' \code{\link[stats]{pnorm}}, the former of which being the most time-consuming
#' operation.
#' }
#' \item{error}{
#' Approximate absolute integration error stemming from the error introduced by
#' the \code{nEval} \code{\link[mvtnorm]{pmvnorm}} evaluations.
#' For this reason, the cubature method is in fact only
#' quasi-exact (as is the \code{pmvnorm} function).
#' }
#' @references
#' Abramowitz, M. and Stegun, I. A. (1972).
#' Handbook of Mathematical Functions with Formulas, Graphs, and Mathematical
#' Tables. New York: Dover Publications.
#' @keywords math spatial
#' @seealso \code{\link{circleCub.Gauss}} for quasi-exact cubature of the
#' isotropic Gaussian density over a circular domain.
#' @family polyCub-methods
#' @example examples/setting.R
#' @examples
#' ## quasi-exact integration based on gpclib::tristrip() and mvtnorm::pmvnorm()
#' \dontrun{## (this example requires gpclib)
#' hexagon.gpc <- new("gpc.poly", pts = lapply(hexagon, c, list(hole = FALSE)))
#' plotpolyf(hexagon.gpc, f, xlim = c(-8,8), ylim = c(-8,8))
#' print(polyCub.exact.Gauss(hexagon.gpc, mean = c(0,0), Sigma = 5^2*diag(2),
#'                           plot = TRUE), digits = 16)
#' }
#' @import methods
#' @importFrom sp plot
#' @importFrom stats cov2cor
#' @importFrom graphics lines
#' @export

polyCub.exact.Gauss <- function (polyregion, mean = c(0,0), Sigma = diag(2),
                                 plot = FALSE)
{
    ## defunctify with a maintainer-level backdoor for building the vignette
    if (!identical(Sys.getenv("R_GPCLIBPERMIT"), "true"))
    .Defunct(msg = paste0(
        "'polyCub.exact.Gauss' is currently unavailable.\n",
        "Contributions are welcome: <https://github.com/bastistician/polyCub/issues/2>"
        ))

    if (inherits(polyregion, "owin")) {
        polyregion <- owin2gpc(polyregion)
    } else if (inherits(polyregion, "sfg")) {
        polyregion <- sfg2gpc(polyregion)
    } else if (!inherits(polyregion, "gpc.poly")) {
        polyregion <- as(polyregion, "gpc.poly")
    }

    stopifnot(is.numeric(mean), length(mean) == 2L, !is.na(mean),
              is.matrix(Sigma), identical(dim(Sigma), c(2L, 2L)),
              is.numeric(Sigma), diag(Sigma) > 0, !is.na(Sigma))

    ## coordinate transformation so that the standard bivariat normal density
    ## can be used in integrations (cf. formula 26.3.22)
    polyregion@pts <- transform_pts(polyregion@pts, mean = mean, Sigma = Sigma)

    ## triangulation: tristrip() returns a list where each element is a
    ## coordinate matrix of vertices of triangles
    ## FIXME: need a reliable tristrip() alternative
    triangleSets <- utils::getFromNamespace("tristrip", "gpclib")(polyregion)

### ILLUSTRATION ###
    if (plot) {
        plot(polyregion, poly.args=list(lwd=2), ann=FALSE)
        lapply(triangleSets, lines, lty=2)
    }
####################

    integrals <- vapply(X = triangleSets, FUN = function (triangles) {
        int <- 0
        error <- 0
        nTriangles <- nrow(triangles) - 2L
        for (i in seq_len(nTriangles)) {
            res <- .intTriangleAS(triangles[i+(0:2),])
            int <- int + res
            error <- error + attr(res, "error")
        }
        c(int, nTriangles, error)
    }, FUN.VALUE = numeric(3L), USE.NAMES = FALSE)
    int <- sum(integrals[1,])

    ## number of .V() evaluations (if there were no degenerate triangles)
    attr(int, "nEval") <- 6 * sum(integrals[2,])
    ## approximate absolute integration error
    attr(int, "error") <- sum(integrals[3,])
    return(int)
}



###########################
### Auxiliary Functions ###
###########################

## transform coordinates according to Formula 26.3.22
transform_pts <- function (pts, mean, Sigma)
{
    mx <- mean[1L]
    my <- mean[2L]
    stopifnot(abs(rho <- cov2cor(Sigma)[1L,2L]) < 1)
    sdx <- sqrt(Sigma[1L,1L])
    sdy <- sqrt(Sigma[2L,2L])
    lapply(pts, function (poly) {
        x0 <- (poly[["x"]] - mx) / sdx
        y0 <- (poly[["y"]] - my) / sdy
        list(x = (x0 + y0) / sqrt(2 + 2*rho),
             y = (y0 - x0) / sqrt(2 - 2*rho),
             hole = poly[["hole"]])
    })
}

## calculates the integral of the standard bivariat normal over a triangle ABC
.intTriangleAS <- function (xy)
{
    if (anyDuplicated(xy)) # degenerate triangle
        return(structure(0, error = 0))
    A <- xy[1,]
    B <- xy[2,]
    C <- xy[3,]
    intAOB <- .intTriangleAS0(A, B)
    intBOC <- .intTriangleAS0(B, C)
    intAOC <- .intTriangleAS0(A, C)

    # determine signs of integrals
    signAOB <- -1 + 2*.pointsOnSameSide(A,B,C)
    signBOC <- -1 + 2*.pointsOnSameSide(B,C,A)
    signAOC <- -1 + 2*.pointsOnSameSide(A,C,B)

    int <- signAOB*intAOB + signBOC*intBOC + signAOC*intAOC
    attr(int, "error") <- attr(intAOB, "error") +
        attr(intBOC, "error") + attr(intAOC, "error")
    return(int)
}

## calculates the integral of the standard bivariat normal over a triangle A0B
.intTriangleAS0 <- function (A, B)
{
    BmA <- B - A
    d <- vecnorm(BmA)
    h <- abs(B[2L]*A[1L] - A[2L]*B[1L]) / d   # distance of AB to the origin
    if (d == 0 || h == 0) # degenerate triangle: A == B or 0, A, B on a line
        return(structure(0, error = 0))

    k1 <- dotprod(A, BmA) / d
    k2 <- dotprod(B, BmA) / d
    V2 <- .V(h, abs(k2))
    V1 <- .V(h, abs(k1))
    res <- if (sign(k1) == sign(k2)) {
        ## A and B are on the same side of the normal line through 0
        abs(V2 - V1)
    } else {
        V2 + V1
    }
    attr(res, "error") <- attr(V1, "error") + attr(V2, "error")
    return(res)
}

## checks if point1 and point2 lie on the same side of a line through
## linepoint1 and linepoint2
.pointsOnSameSide <- function (linepoint1, linepoint2, point1, point2 = c(0,0))
{
    n <- c(-1,1) * rev(linepoint2-linepoint1)   # normal vector
    S <- dotprod(point1-linepoint1,n) * dotprod(point2-linepoint1,n)
    return(S > 0)
}

## calculates the integral of the standard bivariat normal
## over a triangle bounded by y=0, y=ax, x=h (cf. formula 26.3.23)
#' @importFrom stats pnorm
.V <- function(h,k) {
    if (k == 0) # degenerate triangle
        return(structure(0, error = 0))
    a <- k/h
    rho <- -a/sqrt(1+a^2)
    # V = 0.25 + L(h,0,rho) - L(0,0,rho) - Q(h) / 2
    # L(0,0,rho) = 0.25 + asin(rho) / (2*pi)
    # V = L(h,0,rho) - asin(rho)/(2*pi) - Q(h) / 2
    Lh0rho <- mvtnorm::pmvnorm(
        lower = c(h,0), upper = c(Inf,Inf),
        mean = c(0,0), corr = matrix(c(1,rho,rho,1), 2L, 2L)
    )
    Qh <- pnorm(h, mean = 0, sd = 1, lower.tail = FALSE)
    return(Lh0rho - asin(rho)/2/pi - Qh/2)
}

if (!requireNamespace("statmod")) q("no")
library("polyCub")

## statmod::gauss.quad() gives cached Gauss-Legendre nodes/weights
new.NWGL <- lapply(
    X = seq_len(61L),
    FUN = function (n)
    unname(statmod::gauss.quad(n = n, kind = "legendre"))
)
stopifnot(all.equal(new.NWGL, polyCub:::.NWGL, check.attributes = FALSE))

## polyCub.SV() can fetch nodes and weights from 'statmod'
diamond <- list(list(x = c(1,2,1,0), y = c(1,2,3,2)))
nw <- polyCub.SV(diamond, f = NULL, nGQ = 83)  # nGQ > 61
stopifnot(is.list(nw))

## polyCub.SV() can reduce nodes with zero weight
rectangle <- list(list(x = c(-1,1,1,-1), y = c(1,1,2,2)))
##nw0 <- polyCub.SV(rectangle, f = NULL, nGQ = 3, engine = "C")[[1]]  # 0s
nw <- polyCub.SV(rectangle, f = NULL, nGQ = 3, engine = "C+reduce")[[1]]
stopifnot(nw$weights != 0)
##f <- function (s) 1  # => calculate area (= 2)
stopifnot(all.equal(sum(nw$weights), 2))

---
title: "Getting started with 'polyCub'"
author: "Sebastian Meyer"
date: "`{r} Sys.Date()`"
lang: "en"
## litedown:::add_citation() does not currently support R citation files ...
#bibliography: '`{r} system.file("REFERENCES.R", package = "polyCub", mustWork = TRUE)`'
vignette: >
  %\VignetteIndexEntry{Getting started with 'polyCub'}
  %\VignetteEngine{litedown::vignette}
  %\VignetteDepends{polyCub, spatstat.geom}
output:
  html:
    meta:
      js: ["@center-img"]
---

```{r, setup, include = FALSE, purl = FALSE}
litedown::reactor(collapse = TRUE, fig.width = 4, fig.height = 3.5,
                  dev.args = list(units = "in", res = 72))
## kludge: achieve 'echo = -1' by filtering a chunk's "xfun_record_results"
source_drop1 <- function (res)
    replace(res, 1, list(`attributes<-`(res[[1L]][-1], attributes(res[[1]]))))

## maintainer-mode options for building the vignette:
USE_GPCLIB <- identical(Sys.getenv("R_GPCLIBPERMIT"), "true") &&
    requireNamespace("gpclib") # unavailable in --as-cran checks
DO_BENCHMARK <- USE_PNGQUANT <-
    identical(Sys.getenv("NOT_CRAN"), "true")
if (DO_BENCHMARK)
    stopifnot(requireNamespace("microbenchmark"), USE_GPCLIB)
if (USE_PNGQUANT)
    stopifnot(nzchar(Sys.which("pngquant")))
```

<img src="../man/figures/logo.png" align="right" alt="" width="120" />

The R package **polyCub** implements *cubature* (numerical integration)
over *polygonal* domains.
It solves the problem of integrating a continuously differentiable
function f(x,y) over simple closed polygons.

For the special case of a rectangular domain along the axes, the package
[**cubature**](https://CRAN.R-project.org/package=cubature)
is more appropriate (cf.
[`CRAN Task View: Numerical Mathematics`](https://CRAN.R-project.org/view=NumericalMathematics)).


## Polygon representations

The integration domain is described by a polygonal boundary
(or multiple polygons, including holes).
Various R packages for spatial data analysis provide classes for polygons.
The implementations differ in vertex order (which direction represents a hole)
and if the first vertex is repeated.

All of **polyCub**'s cubature methods understand

* `"owin"` from package [**spatstat.geom**](https://CRAN.R-project.org/package=spatstat.geom),

* `"gpc.poly"` from [**gpclib**](https://github.com/rdpeng/gpclib/),

* `"SpatialPolygons"` from package [**sp**](https://CRAN.R-project.org/package=sp), and

* `"(MULTI)POLYGON"` from package [**sf**](https://CRAN.R-project.org/package=sf).

Internally, **polyCub** uses its auxiliary `xylist()` function to extract
a plain list of lists of vertex coordinates from these classes,
such that vertices are ordered anticlockwise (for exterior boundaries)
and the first vertex is not repeated (i.e., the `"owin"` convention).


## Cubature methods

The following cubature methods are implemented in **polyCub**:

1. `polyCub.SV()`: Product Gauss cubature

2. `polyCub.midpoint()`: Two-dimensional midpoint rule
  
3. `polyCub.iso()`: Adaptive cubature for radially symmetric functions
   $f(x,y) = f_r(\lVert(x-x_0,y-y_0)\rVert)$

4. `polyCub.exact.Gauss()` (*currently disabled*): Accurate integration of the
   bivariate Gaussian density

The following section details and illustrates the different cubature methods.


## Illustrations

```{r}
library("polyCub")
```

We consider the integration of a function f(x,y) which all of the above
cubature methods can handle: an isotropic zero-mean Gaussian density.
**polyCub** expects the integrand f to take a two-column
coordinate matrix as its first argument (as opposed to separate arguments
for the x and y coordinates), so:

```{r, example-f}
f <- function (s, sigma = 5)
{
    exp(-rowSums(s^2)/2/sigma^2) / (2*pi*sigma^2)
}
```

We use a simple hexagon as polygonal integration domain,
here specified via an `"xylist"` of vertex coordinates:

```{r, example-polygon}
hexagon <- list(
    list(x = c(7.33, 7.33, 3, -1.33, -1.33, 3),
         y = c(-0.5, 4.5, 7, 4.5, -0.5, -3))
)
```

An image of the function and the integration domain can be produced using
**polyCub**'s rudimentary (but convenient) plotting utility:

```{r, example}
plotpolyf(hexagon, f, xlim = c(-8,8), ylim = c(-8,8))
```


### 1. Product Gauss cubature: `polyCub.SV()`

The **polyCub** package provides an R-interfaced C-translation of
`polygauss.m`, an algorithm by Sommariva and Vianello (2007,
*BIT Numerical Mathematics*, <https://doi.org/10.1007/s10543-007-0131-2>).
The cubature rule is based on Green's integration formula and incorporates
appropriately transformed weights and nodes of one-dimensional
Gauss-Legendre quadrature in both dimensions,
thus the name "product Gauss cubature".
It is exact for all bivariate polynomials if the number of cubature nodes
is sufficiently large (depending on the degree of the polynomial).

For the above example, a reasonable approximation is already obtained
with degree `nGQ = 3` of the one-dimensional Gauss-Legendre quadrature:

```{r, product-Gauss, filter = source_drop1}
par(mar = c(3,3,1,2))
polyCub.SV(hexagon, f, nGQ = 3, plot = TRUE)
```

The involved nodes (displayed in the figure above) and weights can be
extracted by calling `polyCub.SV()` with `f = NULL`, e.g., to determine
the number of nodes:
```{r}
nrow(polyCub.SV(hexagon, f = NULL, nGQ = 3)[[1]]$nodes)
```

For illustration, we create a variant of `polyCub.SV()`,
which returns the number of function evaluations as an attribute:

```{r}
polyCub.SVn <- function (polyregion, f, ..., nGQ = 20) {
    nw <- polyCub.SV(polyregion, f = NULL, ..., nGQ = nGQ)
    ## nw is a list with one element per polygon of 'polyregion'
    res <- sapply(nw, function (x)
        c(result = sum(x$weights * f(x$nodes, ...)), nEval = nrow(x$nodes)))
    structure(sum(res["result",]), nEval = sum(res["nEval",]))
}
polyCub.SVn(hexagon, f, nGQ = 3)
```

We can use this function to investigate how the accuracy of the
approximation depends on the degree `nGQ` and the associated number of
cubature nodes:

```{r}
for (nGQ in c(1:5, 10, 20)) {
    result <- polyCub.SVn(hexagon, f, nGQ = nGQ)
    cat(sprintf("nGQ = %2i: %.12f (n=%i)\n", nGQ, result, attr(result, "nEval")))
}
```


### 2. Two-dimensional midpoint rule: `polyCub.midpoint()`

The two-dimensional midpoint rule in **polyCub** is a simple wrapper
around `as.im.function()` and `integral.im()` from package **spatstat.geom**.
In other words, the polygon is represented by a binary pixel image and
the integral is approximated as the sum of (pixel area * f(pixel midpoint))
over all pixels whose midpoint is part of the polygon.

To use `polyCub.midpoint()`, we need to convert our polygon to
**spatstat.geom**'s `"owin"` class:

```{r, message = FALSE}
library("spatstat.geom")
hexagon.owin <- owin(poly = hexagon)
```

Using a pixel size of `eps = 0.5` (here yielding 270 pixels), we obtain:

```{r, midpoint, filter = source_drop1}
par(mar = c(3,3,1,3), xaxs = "i", yaxs = "i")
polyCub.midpoint(hexagon.owin, f, eps = 0.5, plot = TRUE)
```


### 3. Adaptive cubature for *isotropic* functions: `polyCub.iso()`

A radially symmetric function can be expressed in terms of
the distance r from its point of symmetry: f(r).
If the antiderivative of r times f(r), called `intrfr()`, is
analytically available, Green's theorem leads us to a cubature rule
which only needs *one-dimensional* numerical integration.
More specifically, `intrfr()` will be `integrate()`d along the edges of
the polygon. The mathematical details are given in
Meyer and Held (2014, *The Annals of Applied Statistics*,
<https://doi.org/10.1214/14-AOAS743>, Supplement B, Section 2.4).

For the bivariate Gaussian density `f` defined above,
the integral from 0 to R of `r*f(r)` is analytically available as:
```{r}
intrfr <- function (R, sigma = 5)
{
    (1 - exp(-R^2/2/sigma^2))/2/pi
}
```

With this information, we can apply the cubature rule as follows:

```{r}
polyCub.iso(hexagon, intrfr = intrfr, center = c(0,0))
```

Note that we do not even need the original function `f`.

If `intrfr()` is missing, it can be approximated numerically using
`integrate()` for `r*f(r)` as well, but the overall integration will then
be much less efficient than product Gauss cubature.

Package **polyCub** exposes a C-version of `polyCub.iso()`
for use by other R packages (notably
[**surveillance**](https://CRAN.R-project.org/package=surveillance)) via
`LinkingTo: polyCub` and `#include <polyCubAPI.h>`.
This requires the `intrfr()` function to be implemented in C as well. See
<https://github.com/bastistician/polyCub/blob/master/tests/polyiso_powerlaw.c>
for an example.


### 4. Integration of the *bivariate Gaussian density*: `polyCub.exact.Gauss()`

*This cubature method is currently disabled*
([#2](https://github.com/bastistician/polyCub/issues/2)).
It requires polygon triangulation originally performed using `tristrip()`
from the [**gpclib**](https://github.com/rdpeng/gpclib/) package;
unfortunately, it has become unavailable from mainstream repositories.

Abramowitz and Stegun (1972, Section 26.9, Example 9) offer a formula for
the integral of the bivariate Gaussian density over a triangle with one
vertex at the origin. This formula can be used after triangulation of
the polygonal domain.
The core of the formula is an integral of the bivariate Gaussian density
with zero mean, unit variance and some correlation over an infinite rectangle
[h, Inf] x [0, Inf], which can be computed accurately using `pmvnorm()`
from the [**mvtnorm**](https://CRAN.R-project.org/package=mvtnorm) package.

For the above example, we obtained:

```{r, purl = FALSE, eval = USE_GPCLIB}
polyCub.exact.Gauss(hexagon.owin, mean = c(0,0), Sigma = 5^2*diag(2))
```

The required triangulation as well as the numerous calls of `pmvnorm()`
make this integration algorithm quite cumbersome. For large-scale
integration tasks, it is thus advisable to resort to the general-purpose
product Gauss cubature rule `polyCub.SV()`.

Note: **polyCub** provides an auxiliary function `circleCub.Gauss()` to
calculate the integral of an *isotropic* Gaussian density over a *circular*
domain (which requires nothing more than a single call of `pchisq()`).


## Benchmark

We use the last result from `polyCub.exact.Gauss()` as a reference value and
tune the number of cubature nodes in `polyCub.SV()` and `polyCub.midpoint()`
until the absolute error is below $10^{-8}$.
This leads to `nGQ = 4` for product Gauss cubature
and a 1200 x 1200 pixel image for the midpoint rule.
For `polyCub.iso()`, we keep the default tolerance levels of `integrate()`.
For comparison, we also run `polyCub.iso()` without the analytically derived
`intrfr` function, which leads to a double-`integrate` approximation.

The median runtimes [ms] of the different cubature methods are given below.

```{r, benchmark, purl = FALSE, eval = DO_BENCHMARK}
benchmark <- microbenchmark::microbenchmark(
  SV = polyCub.SV(hexagon.owin, f, nGQ = 4),
  midpoint = polyCub.midpoint(hexagon.owin, f, dimyx = 1200),
  iso = polyCub.iso(hexagon.owin, intrfr = intrfr, center = c(0,0)),
  iso_double_approx = polyCub.iso(hexagon.owin, f, center = c(0,0)),
  exact = polyCub.exact.Gauss(hexagon.owin, mean = c(0,0), Sigma = 5^2*diag(2)),
  times = 9,
  check = function (values) all(abs(unlist(values) - 0.274144773813434) < 1e-8))
```
```{r, purl = FALSE, eval = DO_BENCHMARK, print.args = I(list(digits = 2))}
summary(benchmark, unit = "ms")[c("expr", "median")]
```

The general-purpose SV-method is the clear winner of this small competition.
A disadvantage of that method is that the number of cubature nodes needs to be
tuned manually. This also holds for the midpoint rule, which is by far the
slowest option. In contrast, the "iso"-method for radially symmetric functions
is based on R's `integrate()` function, which implements automatic tolerance
levels. Furthermore, the "iso"-method can also be used with "spiky" integrands,
such as a heavy-tailed power-law kernel $f(r) = (r+1)^{-2}$.



```{r, compact-PNGs, include = FALSE, purl = FALSE, eval = USE_PNGQUANT}
system2("pngquant",
        c("--skip-if-larger", "--speed=1", "--ext=.png", "--force",
          "--", shQuote(litedown::get_context("plot_files"))))
```

---
title: "Getting started with polyCub"
author: "Sebastian Meyer"
date: "`r Sys.Date()`"
output:
  markdown::html_format:
    options:
      smartypants: false
---

<!--
%\VignetteIndexEntry{Getting started with polyCub}
%\VignetteEngine{knitr::knitr}
%\VignetteDepends{polyCub, spatstat.geom, markdown}
%%                                        ^ implied by the engine
-->

```{R setup, include = FALSE, purl = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>",
                      fig.width = 4, fig.height = 3.5)

## maintainer-mode options for building the vignette:
if (USE_GPCLIB <- identical(Sys.getenv("R_GPCLIBPERMIT"), "true")) {
    if (!requireNamespace("gpclib")) # unavailable in --as-cran checks
        USE_GPCLIB <- FALSE
}
if (identical(Sys.getenv("NOT_CRAN"), "true")) {
    stopifnot(nzchar(Sys.which("pngquant"))) # to notice when building
    knitr::knit_hooks$set(pngquant = knitr::hook_pngquant)
    knitr::opts_chunk$set(pngquant = "--speed=1")
    DO_BENCHMARK <- USE_GPCLIB
} else {
    DO_BENCHMARK <- FALSE
}
if (DO_BENCHMARK)
    stopifnot(requireNamespace("microbenchmark"))
```

<img src="../man/figures/logo.png" align="right" alt="" width="120" />

The R package **polyCub** implements *cubature* (numerical integration)
over *polygonal* domains.
It solves the problem of integrating a continuously differentiable
function f(x,y) over simple closed polygons.

For the special case of a rectangular domain along the axes, the package
[**cubature**](https://CRAN.R-project.org/package=cubature)
is more appropriate (cf.
[`CRAN Task View: Numerical Mathematics`](https://CRAN.R-project.org/view=NumericalMathematics)).


## Polygon representations

The integration domain is described by a polygonal boundary
(or multiple polygons, including holes).
Various R packages for spatial data analysis provide classes for polygons.
The implementations differ in vertex order (which direction represents a hole)
and if the first vertex is repeated.

All of **polyCub**'s cubature methods understand

* `"owin"` from package [**spatstat.geom**](https://CRAN.R-project.org/package=spatstat.geom),

* `"gpc.poly"` from [**gpclib**](https://github.com/rdpeng/gpclib/),

* `"SpatialPolygons"` from package [**sp**](https://CRAN.R-project.org/package=sp), and

* `"(MULTI)POLYGON"` from package [**sf**](https://CRAN.R-project.org/package=sf).

Internally, **polyCub** uses its auxiliary `xylist()` function to extract
a plain list of lists of vertex coordinates from these classes,
such that vertices are ordered anticlockwise (for exterior boundaries)
and the first vertex is not repeated (i.e., the `"owin"` convention).


## Cubature methods

The following cubature methods are implemented in **polyCub**:

1. `polyCub.SV()`: Product Gauss cubature

2. `polyCub.midpoint()`: Two-dimensional midpoint rule
  
3. `polyCub.iso()`: Adaptive cubature for radially symmetric functions
   $f(x,y) = f_r(\lVert(x-x_0,y-y_0)\rVert)$

4. `polyCub.exact.Gauss()` (*currently disabled*): Accurate integration of the
   bivariate Gaussian density

The following section details and illustrates the different cubature methods.


## Illustrations

```{R}
library("polyCub")
```

We consider the integration of a function f(x,y) which all of the above
cubature methods can handle: an isotropic zero-mean Gaussian density.
**polyCub** expects the integrand f to take a two-column
coordinate matrix as its first argument (as opposed to separate arguments
for the x and y coordinates), so:

```{R example-f}
f <- function (s, sigma = 5)
{
    exp(-rowSums(s^2)/2/sigma^2) / (2*pi*sigma^2)
}
```

We use a simple hexagon as polygonal integration domain,
here specified via an `"xylist"` of vertex coordinates:

```{R example-polygon}
hexagon <- list(
    list(x = c(7.33, 7.33, 3, -1.33, -1.33, 3),
         y = c(-0.5, 4.5, 7, 4.5, -0.5, -3))
)
```

An image of the function and the integration domain can be produced using
**polyCub**'s rudimentary (but convenient) plotting utility:

```{R example}
plotpolyf(hexagon, f, xlim = c(-8,8), ylim = c(-8,8))
```


### 1. Product Gauss cubature: `polyCub.SV()`

The **polyCub** package provides an R-interfaced C-translation of
"polygauss: Matlab code for Gauss-like cubature over polygons"
(Sommariva and Vianello, 2013, <https://www.math.unipd.it/~alvise/software.html>),
an algorithm described in Sommariva and Vianello (2007,
*BIT Numerical Mathematics*, <https://doi.org/10.1007/s10543-007-0131-2>).
The cubature rule is based on Green's integration formula and incorporates
appropriately transformed weights and nodes of one-dimensional
Gauss-Legendre quadrature in both dimensions,
thus the name "product Gauss cubature".
It is exact for all bivariate polynomials if the number of cubature nodes
is sufficiently large (depending on the degree of the polynomial).

For the above example, a reasonable approximation is already obtained
with degree `nGQ = 3` of the one-dimensional Gauss-Legendre quadrature:

```{R product-Gauss, echo = -1, fig.show = "hold"}
par(mar = c(3,3,1,2))
polyCub.SV(hexagon, f, nGQ = 3, plot = TRUE)
```

The involved nodes (displayed in the figure above) and weights can be
extracted by calling `polyCub.SV()` with `f = NULL`, e.g., to determine
the number of nodes:
```{R}
nrow(polyCub.SV(hexagon, f = NULL, nGQ = 3)[[1]]$nodes)
```

For illustration, we create a variant of `polyCub.SV()`,
which returns the number of function evaluations as an attribute:

```{R}
polyCub.SVn <- function (polyregion, f, ..., nGQ = 20) {
    nw <- polyCub.SV(polyregion, f = NULL, ..., nGQ = nGQ)
    ## nw is a list with one element per polygon of 'polyregion'
    res <- sapply(nw, function (x)
        c(result = sum(x$weights * f(x$nodes, ...)), nEval = nrow(x$nodes)))
    structure(sum(res["result",]), nEval = sum(res["nEval",]))
}
polyCub.SVn(hexagon, f, nGQ = 3)
```

We can use this function to investigate how the accuracy of the
approximation depends on the degree `nGQ` and the associated number of
cubature nodes:

```{R}
for (nGQ in c(1:5, 10, 20)) {
    result <- polyCub.SVn(hexagon, f, nGQ = nGQ)
    cat(sprintf("nGQ = %2i: %.12f (n=%i)\n", nGQ, result, attr(result, "nEval")))
}
```


### 2. Two-dimensional midpoint rule: `polyCub.midpoint()`

The two-dimensional midpoint rule in **polyCub** is a simple wrapper
around `as.im.function()` and `integral.im()` from package **spatstat.geom**.
In other words, the polygon is represented by a binary pixel image and
the integral is approximated as the sum of (pixel area * f(pixel midpoint))
over all pixels whose midpoint is part of the polygon.

To use `polyCub.midpoint()`, we need to convert our polygon to
**spatstat.geom**'s "owin" class:

```{R, message = FALSE}
library("spatstat.geom")
hexagon.owin <- owin(poly = hexagon)
```

Using a pixel size of `eps = 0.5` (here yielding 270 pixels), we obtain:

```{R midpoint, echo = -1, fig.show = "hold"}
par(mar = c(3,3,1,3), xaxs = "i", yaxs = "i")
polyCub.midpoint(hexagon.owin, f, eps = 0.5, plot = TRUE)
```


### 3. Adaptive cubature for *isotropic* functions: `polyCub.iso()`

A radially symmetric function can be expressed in terms of
the distance r from its point of symmetry: f(r).
If the antiderivative of r times f(r), called `intrfr()`, is
analytically available, Green's theorem leads us to a cubature rule
which only needs *one-dimensional* numerical integration.
More specifically, `intrfr()` will be `integrate()`d along the edges of
the polygon. The mathematical details are given in
Meyer and Held (2014, *The Annals of Applied Statistics*,
<https://doi.org/10.1214/14-AOAS743>, Supplement B, Section 2.4).

For the bivariate Gaussian density `f` defined above,
the integral from 0 to R of `r*f(r)` is analytically available as:
```{R}
intrfr <- function (R, sigma = 5)
{
    (1 - exp(-R^2/2/sigma^2))/2/pi
}
```

With this information, we can apply the cubature rule as follows:

```{R}
polyCub.iso(hexagon, intrfr = intrfr, center = c(0,0))
```

Note that we do not even need the original function `f`.

If `intrfr()` is missing, it can be approximated numerically using
`integrate()` for `r*f(r)` as well, but the overall integration will then
be much less efficient than product Gauss cubature.

Package **polyCub** exposes a C-version of `polyCub.iso()`
for use by other R packages (notably
[**surveillance**](https://CRAN.R-project.org/package=surveillance)) via
`LinkingTo: polyCub` and `#include <polyCubAPI.h>`.
This requires the `intrfr()` function to be implemented in C as well. See
<https://github.com/bastistician/polyCub/blob/master/tests/polyiso_powerlaw.c>
for an example.


### 4. Integration of the *bivariate Gaussian density*: `polyCub.exact.Gauss()`

*This cubature method is currently disabled*
([#2](https://github.com/bastistician/polyCub/issues/2)).
It requires polygon triangulation originally performed using `tristrip()`
from the [**gpclib**](https://github.com/rdpeng/gpclib/) package;
unfortunately, it has become unavailable from mainstream repositories.

Abramowitz and Stegun (1972, Section 26.9, Example 9) offer a formula for
the integral of the bivariate Gaussian density over a triangle with one
vertex at the origin. This formula can be used after triangulation of
the polygonal domain.
The core of the formula is an integral of the bivariate Gaussian density
with zero mean, unit variance and some correlation over an infinite rectangle
[h, Inf] x [0, Inf], which can be computed accurately using `pmvnorm()`
from the [**mvtnorm**](https://CRAN.R-project.org/package=mvtnorm) package.

For the above example, we obtained:

```{R, purl = FALSE, eval = USE_GPCLIB}
polyCub.exact.Gauss(hexagon.owin, mean = c(0,0), Sigma = 5^2*diag(2))
```

The required triangulation as well as the numerous calls of `pmvnorm()`
make this integration algorithm quite cumbersome. For large-scale
integration tasks, it is thus advisable to resort to the general-purpose
product Gauss cubature rule `polyCub.SV()`.

Note: **polyCub** provides an auxiliary function `circleCub.Gauss()` to
calculate the integral of an *isotropic* Gaussian density over a *circular*
domain (which requires nothing more than a single call of `pchisq()`).


## Benchmark

We use the last result from `polyCub.exact.Gauss()` as a reference value and
tune the number of cubature nodes in `polyCub.SV()` and `polyCub.midpoint()`
until the absolute error is below $10^{-8}$.
This leads to `nGQ = 4` for product Gauss cubature
and a 1200 x 1200 pixel image for the midpoint rule.
For `polyCub.iso()`, we keep the default tolerance levels of `integrate()`.
For comparison, we also run `polyCub.iso()` without the analytically derived
`intrfr` function, which leads to a double-`integrate` approximation.

The median runtimes [ms] of the different cubature methods are given below.

```{r benchmark, purl = FALSE, eval = DO_BENCHMARK}
benchmark <- microbenchmark::microbenchmark(
  SV = polyCub.SV(hexagon.owin, f, nGQ = 4),
  midpoint = polyCub.midpoint(hexagon.owin, f, dimyx = 1200),
  iso = polyCub.iso(hexagon.owin, intrfr = intrfr, center = c(0,0)),
  iso_double_approx = polyCub.iso(hexagon.owin, f, center = c(0,0)),
  exact = polyCub.exact.Gauss(hexagon.owin, mean = c(0,0), Sigma = 5^2*diag(2)),
  times = 9,
  check = function (values) all(abs(unlist(values) - 0.274144773813434) < 1e-8))
```
```{r, purl = FALSE, eval = FALSE}
summary(benchmark, unit = "ms")[c("expr", "median")]
```
```{r, purl = FALSE, echo = FALSE, eval = DO_BENCHMARK}
knitr::kable(summary(benchmark, unit = "ms")[c("expr", "median")], digits = 2)
```

The general-purpose SV-method is the clear winner of this small competition.
A disadvantage of that method is that the number of cubature nodes needs to be
tuned manually. This also holds for the midpoint rule, which is by far the
slowest option. In contrast, the "iso"-method for radially symmetric functions
is based on R's `integrate()` function, which implements automatic tolerance
levels. Furthermore, the "iso"-method can also be used with "spiky" integrands,
such as a heavy-tailed power-law kernel $f(r) = (r+1)^{-2}$.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coerce-gpc-methods.R
\name{coerce-gpc-methods}
\alias{coerce-gpc-methods}
\alias{owin2gpc}
\alias{gpc2owin}
\alias{as.owin.gpc.poly}
\title{Conversion between polygonal \code{"owin"} and \code{"gpc.poly"}}
\usage{
owin2gpc(object)

gpc2owin(object, ...)

as.owin.gpc.poly(W, ...)
}
\arguments{
\item{object}{an object of class \code{"gpc.poly"} or \code{"owin"},
respectively.}

\item{...}{further arguments passed to \code{\link[spatstat.geom]{owin}}.}

\item{W}{an object of class \code{"gpc.poly"}.}
}
\value{
The converted polygon of class \code{"gpc.poly"} or \code{"owin"},
respectively. If package \pkg{gpclib} is not available,
\code{owin2gpc} will just return the \code{pts} slot of the
\code{"gpc.poly"} (no formal class) with a warning.
}
\description{
Package \pkg{polyCub} implements converters between the classes
\code{"\link[spatstat.geom:owin.object]{owin}"} of package \CRANpkg{spatstat.geom}
and \code{"gpc.poly"} of package \CRANpkg{gpclib}.
}
\note{
The converter \code{owin2gpc} requires the package
\pkg{gpclib} for the formal class definition of a \code{"gpc.poly"}.
It will produce vertices ordered according to the \pkg{sp} convention,
i.e. clockwise for normal boundaries and anticlockwise for holes, where,
however, the first vertex is \emph{not} repeated!
}
\examples{
\dontshow{if (requireNamespace("spatstat.geom")) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
## use example polygons from
example(plotpolyf, ask = FALSE)
letterR  # a simple "xylist"

letterR.owin <- spatstat.geom::owin(poly = letterR)
letterR.gpc_from_owin <- owin2gpc(letterR.owin)
## warns if "gpclib" is unavailable

if (is(letterR.gpc_from_owin, "gpc.poly")) {
  letterR.xylist_from_gpc <- xylist(letterR.gpc_from_owin)
  stopifnot(all.equal(letterR, lapply(letterR.xylist_from_gpc, `[`, 1:2)))
  letterR.owin_from_gpc <- gpc2owin(letterR.gpc_from_owin)
  stopifnot(all.equal(letterR.owin, letterR.owin_from_gpc))
}
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link{xylist}}
}
\author{
Sebastian Meyer
}
\keyword{methods}
\keyword{spatial}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\docType{package}
\name{polyCub-package}
\alias{polyCub-package}
\title{Cubature over Polygonal Domains}
\description{
The \R package \pkg{polyCub} implements
\emph{cubature} (numerical integration) over \emph{polygonal} domains.
It solves the problem of integrating a continuously differentiable
function \eqn{f(x,y)} over simple closed polygons.
}
\details{
\pkg{polyCub} provides the following cubature methods:
\describe{
\item{\code{\link{polyCub.SV}}:}{
General-purpose \emph{product Gauss cubature}
\bibcitep{sommariva.vianello2007}
}
\item{\code{\link{polyCub.midpoint}}:}{
Simple \emph{two-dimensional midpoint rule} based on
\code{\link[spatstat.geom]{as.im.function}} from \CRANpkg{spatstat.geom}
\bibcitep{R:spatstat.geom}
}
\item{\code{\link{polyCub.iso}}:}{
Adaptive cubature for \emph{radially symmetric functions}
via line \code{\link{integrate}()} along the polygon boundary
\bibcitep{|meyer.held2014|Supplement B, Section 2.4}
}
}
A brief description and benchmark experiment of the above cubature
methods can be found in the \code{vignette("polyCub")}.

There is also \code{\link{polyCub.exact.Gauss}}, intended to
accurately (but slowly) integrate the \emph{bivariate Gaussian density};
however, this implementation is disabled as of \pkg{polyCub} 0.9.0:
it needs a reliable implementation of polygon triangulation.

\bibcitet{|meyer2010|Section 3.2}
discusses and compares some of these methods.
}
\note{
To cite package \pkg{polyCub} in publications,
please use \code{citation("polyCub")}:

\Sexpr[results=rd,stage=build]{tools::toRd(citation("polyCub"))}
}
\references{
\bibshow{*}
}
\seealso{
\code{vignette("polyCub")}

For the special case of a rectangular domain along the axes
(e.g., a bounding box), the \CRANpkg{cubature} package is more appropriate.
}
\author{
Sebastian Meyer
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/polyCub.exact.Gauss.R
\name{polyCub.exact.Gauss}
\alias{polyCub.exact.Gauss}
\title{Quasi-Exact Cubature of the Bivariate Normal Density (DEFUNCT)}
\usage{
polyCub.exact.Gauss(polyregion, mean = c(0, 0), Sigma = diag(2),
  plot = FALSE)
}
\arguments{
\item{polyregion}{a \code{"gpc.poly"} polygon or
something that can be coerced to this class, e.g., an \code{"owin"} polygon
(via \code{\link{owin2gpc}}), or an \code{"sfg"} polygon (via
\code{\link{sfg2gpc}}).}

\item{mean, Sigma}{mean and covariance matrix of the bivariate normal density
to be integrated.}

\item{plot}{logical indicating if an illustrative plot of the numerical
integration should be produced. Note that the \code{polyregion} will be
transformed (shifted and scaled).}
}
\value{
The integral of the bivariate normal density over \code{polyregion}.
Two attributes are appended to the integral value:
\item{nEval}{
number of triangles over which the standard bivariate normal density had to
be integrated, i.e. number of calls to \code{\link[mvtnorm]{pmvnorm}} and
\code{\link[stats]{pnorm}}, the former of which being the most time-consuming
operation.
}
\item{error}{
Approximate absolute integration error stemming from the error introduced by
the \code{nEval} \code{\link[mvtnorm]{pmvnorm}} evaluations.
For this reason, the cubature method is in fact only
quasi-exact (as is the \code{pmvnorm} function).
}
}
\description{
This cubature method is \strong{defunct} as of \pkg{polyCub} version 0.9.0.
It relied on \code{tristrip()} from package \CRANpkg{gpclib} for polygon
triangulation, but that package did not have a \acronym{FOSS} license and
was no longer maintained on a mainstream repository.\cr
Contributions to resurrect this cubature method are welcome: an alternative
implementation for constrained polygon triangulation is needed, see
\url{https://github.com/bastistician/polyCub/issues/2}.
}
\details{
The bivariate Gaussian density can be integrated based on a triangulation of
the (transformed) polygonal domain, using formulae from the
Abramowitz and Stegun (1972) handbook (Section 26.9, Example 9, pp. 956f.).
This method is quite cumbersome because the A&S formula is only for triangles
where one vertex is the origin (0,0). For each triangle
we have to check in which of the 6 outer
regions of the triangle the origin (0,0) lies and adapt the signs in the
formula appropriately: \eqn{(AOB+BOC-AOC)} or \eqn{(AOB-AOC-BOC)} or
\eqn{(AOB+AOC-BOC)} or \eqn{(AOC+BOC-AOB)} or \ldots.
However, the most time consuming step is the
evaluation of \code{\link[mvtnorm]{pmvnorm}}.
}
\examples{
## a function to integrate (here: isotropic zero-mean Gaussian density)
f <- function (s, sigma = 5)
    exp(-rowSums(s^2)/2/sigma^2) / (2*pi*sigma^2)

## a simple polygon as integration domain
hexagon <- list(
    list(x = c(7.33, 7.33, 3, -1.33, -1.33, 3),
         y = c(-0.5, 4.5, 7, 4.5, -0.5, -3))
)

## quasi-exact integration based on gpclib::tristrip() and mvtnorm::pmvnorm()
\dontrun{## (this example requires gpclib)
hexagon.gpc <- new("gpc.poly", pts = lapply(hexagon, c, list(hole = FALSE)))
plotpolyf(hexagon.gpc, f, xlim = c(-8,8), ylim = c(-8,8))
print(polyCub.exact.Gauss(hexagon.gpc, mean = c(0,0), Sigma = 5^2*diag(2),
                          plot = TRUE), digits = 16)
}
}
\references{
Abramowitz, M. and Stegun, I. A. (1972).
Handbook of Mathematical Functions with Formulas, Graphs, and Mathematical
Tables. New York: Dover Publications.
}
\seealso{
\code{\link{circleCub.Gauss}} for quasi-exact cubature of the
isotropic Gaussian density over a circular domain.

Other polyCub-methods: 
\code{\link{polyCub}()},
\code{\link{polyCub.SV}()},
\code{\link{polyCub.iso}()},
\code{\link{polyCub.midpoint}()}
}
\concept{polyCub-methods}
\keyword{math}
\keyword{spatial}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/polyCub.R
\name{polyCub}
\alias{polyCub}
\title{Wrapper Function for the Various Cubature Methods}
\usage{
polyCub(polyregion, f, method = c("SV", "midpoint", "iso", "exact.Gauss"),
  ..., plot = FALSE)
}
\arguments{
\item{polyregion}{a polygonal domain.
The following classes are supported:
\code{"\link[spatstat.geom]{owin}"} from package \pkg{spatstat.geom},
\code{"gpc.poly"} from \pkg{gpclib},
\code{"\link[sp:SpatialPolygons-class]{SpatialPolygons}"},
\code{"\link[sp:Polygons-class]{Polygons}"},
and \code{"\link[sp:Polygon-class]{Polygon}"} from package \pkg{sp}, as well as
\code{"\link[sf:st_polygon]{(MULTI)POLYGON}"} from package \pkg{sf}.
(For these classes, \pkg{polyCub} knows how to get an \code{\link{xylist}}.)}

\item{f}{a two-dimensional real-valued function to be integrated over
\code{polyregion}. As its first argument it must take a coordinate matrix,
i.e., a numeric matrix with two columns, and it must return a numeric vector
of length the number of coordinates.\cr
For the \code{"exact.Gauss"} \code{method},
\code{f} is ignored since it is specific to the bivariate normal density.}

\item{method}{choose one of the implemented cubature methods (partial
argument matching is applied), see \code{help("\link{polyCub-package}")}
for an overview. Defaults to using product Gauss cubature
implemented in \code{\link{polyCub.SV}}.}

\item{...}{arguments of \code{f} or of the specific \code{method}.}

\item{plot}{logical indicating if an illustrative plot of the numerical
integration should be produced.}
}
\value{
The approximated integral of \code{f} over \code{polyregion}.
}
\description{
The wrapper function \code{polyCub} can be used to call specific cubature
methods via its \code{method} argument. It calls the \code{\link{polyCub.SV}}
function by default, which implements general-purpose product Gauss cubature.
The desired cubature function should usually be called directly.
}
\seealso{
Details and examples in the \code{vignette("polyCub")}
and on the method-specific help pages.

Other polyCub-methods: 
\code{\link{polyCub.SV}()},
\code{\link{polyCub.exact.Gauss}()},
\code{\link{polyCub.iso}()},
\code{\link{polyCub.midpoint}()}
}
\concept{polyCub-methods}
\keyword{math}
\keyword{spatial}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{dotprod}
\alias{dotprod}
\title{Dot/Scalar Product of Two Vectors}
\usage{
dotprod(x, y)
}
\arguments{
\item{x, y}{numeric vectors (of compatible lengths).}
}
\value{
\code{sum(x*y)}
}
\description{
This is nothing else than \code{sum(x*y)}.
}
\keyword{internal}
\keyword{math}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/polyCub.iso.R
\name{checkintrfr}
\alias{checkintrfr}
\title{Check the Integral of \eqn{r f_r(r)}}
\usage{
checkintrfr(intrfr, f, ..., center, control = list(), rs = numeric(0L),
  tolerance = control$rel.tol)
}
\arguments{
\item{intrfr}{a \code{function(R, ...)}, which implements the (analytical)
antiderivative of \eqn{r f_r(r)} from 0 to \code{R}. The first argument
must be vectorized but not necessarily named \code{R}.\cr
If \code{intrfr} is missing, it will be approximated numerically via
\preformatted{
  integrate(function(r, ...) r * f(cbind(x0 + r, y0), ...),
            0, R, ..., control = control)
}
where \code{c(x0, y0)} is the \code{center} of isotropy.
Note that \code{f} will \emph{not} be checked for isotropy.}

\item{f}{a two-dimensional real-valued function.
As its first argument it must take a coordinate matrix, i.e., a
numeric matrix with two columns, and it must return a numeric vector of
length the number of coordinates.}

\item{...}{further arguments for \code{f} or \code{intrfr}.}

\item{center}{numeric vector of length 2, the center of isotropy.}

\item{control}{list of arguments passed to \code{\link{integrate}}, the
quadrature rule used for the line integral along the polygon boundary.}

\item{rs}{numeric vector of upper bounds for which to check the validity of
\code{intrfr}. If it has length 0 (default), no checks are performed.}

\item{tolerance}{of \code{\link{all.equal.numeric}} when comparing
\code{intrfr} results with numerical integration. Defaults to the
relative tolerance used for \code{integrate}.}
}
\value{
The \code{intrfr} function, invisibly. If only \code{f} was given,
an \code{integrate}-based approximation of \code{intrfr} is returned.
}
\description{
This function is auxiliary to \code{\link{polyCub.iso}}
for the cubature of a radially symmetric function
\eqn{f(x,y) = f_r(||(x,y)-\boldsymbol{\mu}||)}{f(x,y) = f_r(||(x,y)-\mu||)},
with \eqn{\mu} being the center of isotropy, over a polygonal domain.
The (analytical) integral of \eqn{r f_r(r)} from 0 to \eqn{R}, \code{intrfr},
is checked against an \code{\link{integrate}}-based approximation
for various values (\code{rs}) of the upper bound \eqn{R}.
A warning is issued if inconsistencies are found.
}
\examples{
f_const <- function (coords) rep(1, nrow(coords))
intrfr_const <- function (R) R^2/2  # = \int_0^R r f_r(r) dr
checkintrfr(intrfr_const,  f = f_const, center = c(0,0), rs = 1:10) # OK
checkintrfr(function(R) R, f = f_const, center = c(0,0), rs = 1:10) # warns
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/polyCub.SV.R
\name{polygauss}
\alias{polygauss}
\title{Calculate 2D Nodes and Weights of the Product Gauss Cubature}
\usage{
polygauss(xy, nw_MN, alpha = NULL, rotation = FALSE, engine = "C")
}
\arguments{
\item{xy}{list with elements \code{"x"} and \code{"y"} containing the
polygon vertices in \emph{anticlockwise} order (otherwise the result of the
cubature will have a negative sign) with first vertex not repeated at the
end (like \code{owin.object$bdry}).}

\item{nw_MN}{unnamed list of nodes and weights of one-dimensional Gauss
quadrature rules of degrees \eqn{N} and \eqn{M=N+1} (as returned by
\code{\link[statmod]{gauss.quad}}): \code{list(s_M, w_M, s_N, w_N)}.}

\item{alpha}{base-line of the (rotated) polygon at \eqn{x = \alpha} (see
Sommariva and Vianello (2007) for an explication). If \code{NULL} (default),
the midpoint of the x-range of each polygon is chosen if no \code{rotation}
is performed, and otherwise the \eqn{x}-coordinate of the rotated point
\code{"P"} (see \code{rotation}). If \code{f} has its maximum value at the
origin \eqn{(0,0)}, e.g., the bivariate Gaussian density with zero mean,
\code{alpha = 0} is a reasonable choice.}

\item{rotation}{logical (default: \code{FALSE}) or a list of points
\code{"P"} and \code{"Q"} describing the preferred direction. If
\code{TRUE}, the polygon is rotated according to the vertices \code{"P"} and
\code{"Q"}, which are farthest apart (see Sommariva and Vianello, 2007). For
convex polygons, this rotation guarantees that all nodes fall inside the
polygon.}

\item{engine}{character string specifying the implementation to use.
Up to \pkg{polyCub} version 0.4-3, the two-dimensional nodes and weights
were computed by \R functions and these are still available by setting
\code{engine = "R"}.
The new C-implementation is now the default (\code{engine = "C"}) and
requires approximately 30\% less computation time.\cr
The special setting \code{engine = "C+reduce"} will discard redundant nodes
at (0,0) with zero weight resulting from edges on the base-line
\eqn{x = \alpha} or orthogonal to it.
This extra cleaning is only worth its cost for computationally intensive
functions \code{f} over polygons which really have some edges on the
baseline or parallel to the x-axis.  Note that the old \R
implementation does not have such unset zero nodes and weights.}
}
\description{
Calculate 2D Nodes and Weights of the Product Gauss Cubature
}
\references{
Sommariva, A. and Vianello, M. (2007):
Product Gauss cubature over polygons based on Green's integration formula.
\emph{BIT Numerical Mathematics}, \bold{47} (2), 441-453.
\doi{10.1007/s10543-007-0131-2}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{isScalar}
\alias{isScalar}
\title{Checks if Argument is Scalar}
\usage{
isScalar(x)
}
\arguments{
\item{x}{any object}
}
\value{
logical
}
\description{
Check if the argument is scalar, i.e. a numeric vector of length 1.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/circleCub.R
\name{circleCub.Gauss}
\alias{circleCub.Gauss}
\title{Integration of the Isotropic Gaussian Density over Circular Domains}
\usage{
circleCub.Gauss(center, r, mean, sd)
}
\arguments{
\item{center}{numeric vector of length 2 (center of the circle).}

\item{r}{numeric (radius of the circle). Several radii may be supplied.}

\item{mean}{numeric vector of length 2
(mean of the bivariate Gaussian density).}

\item{sd}{numeric (common standard deviation of the isotropic
Gaussian density in both dimensions).}
}
\value{
The integral value (one for each supplied radius).
}
\description{
This function calculates the integral of the bivariate, isotropic Gaussian
density (i.e., \eqn{\Sigma} = \code{sd^2*diag(2)}) over a circular domain
via the cumulative distribution function \code{pchisq} of the (non-central)
Chi-Squared distribution \bibcitep{|R:Abramowitz+Stegun:1972|Formula 26.3.24}.
}
\note{
The non-centrality parameter of the evaluated chi-squared distribution
equals the squared distance between the \code{mean} and the
\code{center}. If this becomes too large, the result becomes inaccurate, see
\code{\link{pchisq}}.
}
\examples{
circleCub.Gauss(center=c(1,2), r=3, mean=c(4,5), sd=6)

## compare with cubature over a polygonal approximation of a circle
d2norm <- function (s, mean, sd)
  dnorm(s[,1], mean=mean[1], sd=sd) * dnorm(s[,2], mean=mean[2], sd=sd)
if (requireNamespace("spatstat.geom")) { # for the disc()
  npoly <- 32  # increase this for a closer match
  disc.poly <- spatstat.geom::disc(radius=3, centre=c(1,2), npoly=npoly)
  polyCub.iso(disc.poly, d2norm, mean=c(4,5), sd=6, center=c(4,5))
}
}
\references{
\bibshow{*}
}
\keyword{math}
\keyword{spatial}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\name{gpclibPermit}
\alias{gpclibPermit}
\alias{gpclibPermitStatus}
\title{\pkg{gpclib} License Acceptance (OBSOLETE)}
\usage{
gpclibPermit()

gpclibPermitStatus()
}
\description{
Previous versions of package \CRANpkg{gpclib} had a restricted license
(commercial use prohibited) and these functions were used as a blocker.
They now always return \code{TRUE}.
}
\details{
\pkg{gpclib} functionality is only required for
\code{\link{polyCub.exact.Gauss}}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/polyCub.iso.R
\name{polyCub.iso}
\alias{polyCub.iso}
\alias{.polyCub.iso}
\title{Cubature of Isotropic Functions over Polygonal Domains}
\usage{
polyCub.iso(polyregion, f, intrfr, ..., center, control = list(),
  check.intrfr = FALSE, plot = FALSE)

.polyCub.iso(polys, intrfr, ..., center, control = list(),
  .witherror = FALSE)
}
\arguments{
\item{polyregion}{a polygonal domain.
The following classes are supported:
\code{"\link[spatstat.geom]{owin}"} from package \pkg{spatstat.geom},
\code{"gpc.poly"} from \pkg{gpclib},
\code{"\link[sp:SpatialPolygons-class]{SpatialPolygons}"},
\code{"\link[sp:Polygons-class]{Polygons}"},
and \code{"\link[sp:Polygon-class]{Polygon}"} from package \pkg{sp}, as well as
\code{"\link[sf:st_polygon]{(MULTI)POLYGON}"} from package \pkg{sf}.
(For these classes, \pkg{polyCub} knows how to get an \code{\link{xylist}}.)}

\item{f}{a two-dimensional real-valued function.
As its first argument it must take a coordinate matrix, i.e., a
numeric matrix with two columns, and it must return a numeric vector of
length the number of coordinates.}

\item{intrfr}{a \code{function(R, ...)}, which implements the (analytical)
antiderivative of \eqn{r f_r(r)} from 0 to \code{R}. The first argument
must be vectorized but not necessarily named \code{R}.\cr
If \code{intrfr} is missing, it will be approximated numerically via
\preformatted{
  integrate(function(r, ...) r * f(cbind(x0 + r, y0), ...),
            0, R, ..., control = control)
}
where \code{c(x0, y0)} is the \code{center} of isotropy.
Note that \code{f} will \emph{not} be checked for isotropy.}

\item{...}{further arguments for \code{f} or \code{intrfr}.}

\item{center}{numeric vector of length 2, the center of isotropy.}

\item{control}{list of arguments passed to \code{\link{integrate}}, the
quadrature rule used for the line integral along the polygon boundary.}

\item{check.intrfr}{logical (or numeric vector) indicating if
(for which \code{r}'s) the supplied \code{intrfr} function should be
checked against a numerical approximation. This check requires \code{f}
to be specified. If \code{TRUE}, the set of test
\code{r}'s defaults to a \code{\link{seq}} of length 20 from 1 to
the maximum absolute x or y coordinate of any edge of the \code{polyregion}.}

\item{plot}{logical indicating if an image of the function should be plotted
together with the polygonal domain, i.e.,
\code{\link{plotpolyf}(polyregion, f, \dots)}.}

\item{polys}{something like \code{owin$bdry}, but see \code{\link{xylist}}.}

\item{.witherror}{logical indicating if an upper bound for the absolute
integration error should be attached as an attribute to the result?}
}
\value{
The approximate integral of the isotropic function
\code{f} over \code{polyregion}.\cr
If the \code{intrfr} function is provided (which is assumed to be exact), an
upper bound for the absolute integration error is appended to the result as
attribute \code{"abs.error"}. It equals the sum of the absolute errors
reported by all \code{\link{integrate}} calls
(there is one for each edge of \code{polyregion}).
}
\description{
\code{polyCub.iso} numerically integrates a radially symmetric function
\eqn{f(x,y) = f_r(||(x,y)-\boldsymbol{\mu}||)}{f(x,y) = f_r(||(x,y)-\mu||)},
with \eqn{\mu} being the center of isotropy, over a polygonal domain.
It internally approximates a line integral along the polygon boundary using
\code{\link{integrate}}. The integrand requires the antiderivative of
\eqn{r f_r(r)}), which should be supplied as argument \code{intrfr}
(\code{f} itself is only required if \code{check.intrfr=TRUE}).
The two-dimensional integration problem thereby reduces to an efficient
adaptive quadrature in one dimension.
If \code{intrfr} is not available analytically, \code{polyCub.iso} can use a
numerical approximation (meaning \code{integrate} within \code{integrate}),
but the general-purpose cubature method \code{\link{polyCub.SV}} might be
more efficient in this case.
See \bibcitet{|meyer.held2014|Supplement B, Section 2.4}
for mathematical details.

\code{.polyCub.iso} is a \dQuote{bare-bone} version of \code{polyCub.iso}.
}
\examples{
## we use the example polygon and f (exponential decay) from
example(plotpolyf)

## numerical approximation of 'intrfr' (not recommended)
(intISOnum <- polyCub.iso(letterR, f, center = fcenter))

## analytical 'intrfr'
## intrfr(R) = int_0^R r*f(r) dr, for f(r) = dexp(r), gives
intrfr <- function (R, rate = 1) pgamma(R, 2, rate) / rate
(intISOana <- polyCub.iso(letterR, f, intrfr = intrfr, center = fcenter,
                          check.intrfr = TRUE))
## f is only used to check 'intrfr' against a numerical approximation

stopifnot(all.equal(intISOana, intISOnum, check.attributes = FALSE))


### polygon area: f(r) = 1, f(x,y) = 1, center does not really matter

## intrfr(R) = int_0^R r*f(r) dr = int_0^R r dr = R^2/2
intrfr.const <- function (R) R^2/2
(area.ISO <- polyCub.iso(letterR, intrfr = intrfr.const, center = c(0,0)))

if (require("spatstat.geom")) { # check against area.owin()
    stopifnot(all.equal(area.owin(owin(poly = letterR)),
                        area.ISO, check.attributes = FALSE))
}
}
\references{
\bibshow{*}
}
\seealso{
\code{system.file("include", "polyCubAPI.h", package = "polyCub")}
for a full C-implementation of this cubature method (for a \emph{single}
polygon). The corresponding C-routine \code{polyCub_iso} can be used by
other \R packages, notably \CRANpkg{surveillance}, via \samp{LinkingTo: polyCub}
(in the \file{DESCRIPTION}) and \samp{#include <polyCubAPI.h>} (in suitable
\file{/src} files). Note that the \code{intrfr} function must then also be
supplied as a C-routine. An example can be found in the package tests.

Other cubature methods: 
\code{\link{polyCub}()},
\code{\link{polyCub.SV}()},
\code{\link{polyCub.midpoint}()}
}
\author{
Sebastian Meyer

The basic mathematical formulation of this efficient integration for radially
symmetric functions was ascertained with great support by
Emil Hedevang (Dept. of Mathematics, Aarhus University, Denmark)
during the Summer School on Topics in
Space-Time Modeling and Inference (May 2013, Aalborg, Denmark).
}
\concept{cubature methods}
\keyword{math}
\keyword{spatial}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xylist.R
\name{xylist}
\alias{xylist}
\alias{xylist.owin}
\alias{xylist.sfg}
\alias{xylist.gpc.poly}
\alias{xylist.SpatialPolygons}
\alias{xylist.Polygons}
\alias{xylist.Polygon}
\alias{xylist.default}
\title{Convert Various Polygon Classes to a Simple List of Vertices}
\usage{
xylist(object, ...)

\method{xylist}{owin}(object, ...)

\method{xylist}{sfg}(object, ...)

\method{xylist}{gpc.poly}(object, ...)

\method{xylist}{SpatialPolygons}(object, reverse = TRUE, ...)

\method{xylist}{Polygons}(object, reverse = TRUE, ...)

\method{xylist}{Polygon}(object, reverse = TRUE, ...)

\method{xylist}{default}(object, ...)
}
\arguments{
\item{object}{an object of one of the supported spatial classes.}

\item{...}{(unused) argument of the generic.}

\item{reverse}{logical (\code{TRUE}) indicating if the vertex order of the
\pkg{sp} classes should be reversed to get the \code{xylist}/\code{owin}
convention.}
}
\value{
Applying \code{xylist} to a polygon object, one gets a simple list,
where each component (polygon) is a list of \code{"x"} and \code{"y"}
coordinates. These represent vertex coordinates following \pkg{spatstat.geom}'s
\code{"owin"} convention (anticlockwise order for exterior boundaries,
without repeating any vertex).
}
\description{
Different packages concerned with spatial data use different polygon
specifications, which sometimes becomes very confusing (see Details below).
To be compatible with the various polygon classes, package \pkg{polyCub}
uses an S3 class \code{"xylist"}, which represents a polygonal domain
(of potentially multiple polygons) by its core feature only: a list of lists
of vertex coordinates (see the "Value" section below).
The generic function \code{xylist} can deal with the
following polygon classes:
\itemize{
\item \code{"\link[spatstat.geom:owin.object]{owin}"} from package \pkg{spatstat.geom}
\item \code{"gpc.poly"} from package \pkg{gpclib}
\item \code{"\link[sp:Polygons-class]{Polygons}"} from package \pkg{sp}
(as well as \code{"\link[sp:Polygon-class]{Polygon}"} and
\code{"\link[sp:SpatialPolygons-class]{SpatialPolygons}"})
\item \code{"\link[sf:st_polygon]{(MULTI)POLYGON}"} from package \pkg{sf}
}
The (somehow useless) default \code{xylist}-method
does not perform any transformation but only ensures that the polygons are
not closed (first vertex not repeated).
}
\details{
Polygon specifications differ with respect to:
\itemize{
\item is the first vertex repeated?
\item which ring direction represents holes?
}
Package overview:
\describe{
\item{\pkg{spatstat.geom}:}{\code{"owin"} does \emph{not repeat} the
first vertex, and anticlockwise = normal boundary, clockwise = hole.
This convention is also used for the return value of \code{xylist}.}
\item{\pkg{sp}:}{\emph{Repeat} first vertex at the end (closed),
anticlockwise = hole, clockwise = normal boundary}
\item{\pkg{sf}:}{\emph{Repeat} first vertex at the end (closed),
clockwise = hole, anticlockwise = normal boundary;
\emph{however}, \pkg{sf} does not check the ring direction by default, so
it cannot be relied upon.}
\item{\pkg{gpclib}:}{There seem to be no such conventions
for polygons of class \code{"gpc.poly"}.}
}
Thus, for polygons from \pkg{sf} and \pkg{gpclib}, \code{xylist} needs
to check the ring direction, which makes these two formats the least
efficient for integration domains in \pkg{polyCub}.
}
\author{
Sebastian Meyer
}
\keyword{methods}
\keyword{spatial}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/polyCub.SV.R
\name{polyCub.SV}
\alias{polyCub.SV}
\title{Product Gauss Cubature over Polygonal Domains}
\usage{
polyCub.SV(polyregion, f, ..., nGQ = 20, alpha = NULL, rotation = FALSE,
  engine = "C", plot = FALSE)
}
\arguments{
\item{polyregion}{a polygonal domain.
The following classes are supported:
\code{"\link[spatstat.geom]{owin}"} from package \pkg{spatstat.geom},
\code{"gpc.poly"} from \pkg{gpclib},
\code{"\link[sp:SpatialPolygons-class]{SpatialPolygons}"},
\code{"\link[sp:Polygons-class]{Polygons}"},
and \code{"\link[sp:Polygon-class]{Polygon}"} from package \pkg{sp}, as well as
\code{"\link[sf:st_polygon]{(MULTI)POLYGON}"} from package \pkg{sf}.
(For these classes, \pkg{polyCub} knows how to get an \code{\link{xylist}}.)}

\item{f}{a two-dimensional real-valued function to be integrated over
\code{polyregion} (or \code{NULL} to only compute nodes and weights).
As its first argument it must take a coordinate matrix, i.e., a
numeric matrix with two columns, and it must return a numeric vector of
length the number of coordinates.}

\item{...}{further arguments for \code{f}.}

\item{nGQ}{degree of the one-dimensional Gauss-Legendre quadrature rule
(default: 20) as implemented in function \code{\link[statmod]{gauss.quad}}
of package \CRANpkg{statmod}. Nodes and weights up to \code{nGQ=60} are cached
in \pkg{polyCub}, for larger degrees \pkg{statmod} is required.}

\item{alpha}{base-line of the (rotated) polygon at \eqn{x = \alpha} (see
Sommariva and Vianello (2007) for an explication). If \code{NULL} (default),
the midpoint of the x-range of each polygon is chosen if no \code{rotation}
is performed, and otherwise the \eqn{x}-coordinate of the rotated point
\code{"P"} (see \code{rotation}). If \code{f} has its maximum value at the
origin \eqn{(0,0)}, e.g., the bivariate Gaussian density with zero mean,
\code{alpha = 0} is a reasonable choice.}

\item{rotation}{logical (default: \code{FALSE}) or a list of points
\code{"P"} and \code{"Q"} describing the preferred direction. If
\code{TRUE}, the polygon is rotated according to the vertices \code{"P"} and
\code{"Q"}, which are farthest apart (see Sommariva and Vianello, 2007). For
convex polygons, this rotation guarantees that all nodes fall inside the
polygon.}

\item{engine}{character string specifying the implementation to use.
Up to \pkg{polyCub} version 0.4-3, the two-dimensional nodes and weights
were computed by \R functions and these are still available by setting
\code{engine = "R"}.
The new C-implementation is now the default (\code{engine = "C"}) and
requires approximately 30\% less computation time.\cr
The special setting \code{engine = "C+reduce"} will discard redundant nodes
at (0,0) with zero weight resulting from edges on the base-line
\eqn{x = \alpha} or orthogonal to it.
This extra cleaning is only worth its cost for computationally intensive
functions \code{f} over polygons which really have some edges on the
baseline or parallel to the x-axis.  Note that the old \R
implementation does not have such unset zero nodes and weights.}

\item{plot}{logical indicating if an illustrative plot of the numerical
integration should be produced.}
}
\value{
The approximated value of the integral of \code{f} over
\code{polyregion}.\cr
In the case \code{f = NULL}, only the computed nodes and weights are
returned in a list of length the number of polygons of \code{polyregion},
where each component is a list with \code{nodes} (a numeric matrix with
two columns), \code{weights} (a numeric vector of length
\code{nrow(nodes)}), the rotation \code{angle}, and \code{alpha}.
}
\description{
Product Gauss cubature over polygons as proposed by
Sommariva and Vianello (2007).
}
\examples{
## a function to integrate (here: isotropic zero-mean Gaussian density)
f <- function (s, sigma = 5)
    exp(-rowSums(s^2)/2/sigma^2) / (2*pi*sigma^2)

## a simple polygon as integration domain
hexagon <- list(
    list(x = c(7.33, 7.33, 3, -1.33, -1.33, 3),
         y = c(-0.5, 4.5, 7, 4.5, -0.5, -3))
)

## image of the function and integration domain
plotpolyf(hexagon, f)

## use a degree of nGQ = 3 and show the corresponding nodes
polyCub.SV(hexagon, f, nGQ = 3, plot = TRUE)

## extract nodes and weights
nw <- polyCub.SV(hexagon, f = NULL, nGQ = 3)[[1]]
nrow(nw$nodes)

## manually apply the cubature rule
sum(nw$weights * f(nw$nodes))

## use an increasing number of nodes
for (nGQ in c(1:5, 10, 20, 60))
    cat(sprintf("nGQ = \%2i: \%.16f\n", nGQ,
                polyCub.SV(hexagon, f, nGQ = nGQ)))

## polyCub.SV() is the default method used by the polyCub() wrapper
polyCub(hexagon, f, nGQ = 3)  # calls polyCub.SV()


### now using a simple *rectangular* integration domain

rectangle <- list(list(x = c(-1, 7, 7, -1), y = c(-3, -3, 7, 7)))
polyCub.SV(rectangle, f, plot = TRUE)

## effect of rotation given a very low nGQ
opar <- par(mfrow = c(1,3))
polyCub.SV(rectangle, f, nGQ = 4, rotation = FALSE, plot = TRUE)
           title(main = "without rotation (default)")
polyCub.SV(rectangle, f, nGQ = 4, rotation = TRUE,  plot = TRUE)
           title(main = "standard rotation")
polyCub.SV(rectangle, f, nGQ = 4,
           rotation = list(P = c(0,0), Q = c(2,-3)), plot = TRUE)
           title(main = "custom rotation")
par(opar)

## comparison with the "cubature" package
if (requireNamespace("cubature")) {
    fc <- function (s, sigma = 5)  # non-vectorized version of f
        exp(-sum(s^2)/2/sigma^2) / (2*pi*sigma^2)
    cubature::hcubature(fc, lowerLimit = c(-1, -3), upperLimit = c(7, 7))
}
}
\references{
Sommariva, A. and Vianello, M. (2007):
Product Gauss cubature over polygons based on Green's integration formula.
\emph{BIT Numerical Mathematics}, \bold{47} (2), 441-453.
\doi{10.1007/s10543-007-0131-2}
}
\seealso{
Other polyCub-methods: 
\code{\link{polyCub}()},
\code{\link{polyCub.exact.Gauss}()},
\code{\link{polyCub.iso}()},
\code{\link{polyCub.midpoint}()}
}
\author{
Sebastian Meyer\cr
These R and C implementations of product Gauss cubature are based on the
original \acronym{MATLAB} implementation \code{polygauss} by Sommariva and
Vianello (2007), which is available under the GNU GPL (>=2) license from
\url{https://www.math.unipd.it/~alvise/software.html}.
}
\concept{polyCub-methods}
\keyword{math}
\keyword{spatial}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coerce-sp-methods.R
\name{coerce-sp-methods}
\alias{coerce-sp-methods}
\alias{as.owin.SpatialPolygons}
\alias{as.owin.Polygons}
\alias{as.owin.Polygon}
\alias{coerce,SpatialPolygons,owin-method}
\alias{coerce,Polygons,owin-method}
\alias{coerce,Polygon,owin-method}
\alias{coerce,Polygon,Polygons-method}
\title{Coerce \code{"SpatialPolygons"} to \code{"owin"}}
\usage{
as.owin.SpatialPolygons(W, ...)

as.owin.Polygons(W, ...)

as.owin.Polygon(W, ...)
}
\arguments{
\item{W}{an object of class \code{"SpatialPolygons"},
\code{"Polygons"}, or \code{"Polygon"}.}

\item{...}{further arguments passed to \code{\link[spatstat.geom]{owin}}.}
}
\description{
Package \pkg{polyCub} implements \code{coerce}-methods
(\code{as(object, Class)}) to convert
\code{"\link[sp:SpatialPolygons-class]{SpatialPolygons}"}
(or \code{"\link[sp:Polygons-class]{Polygons}"}
or \code{"\link[sp:Polygon-class]{Polygon}"})
of package \CRANpkg{sp}
to \code{"\link[spatstat.geom:owin.object]{owin}"}
of package \CRANpkg{spatstat.geom}.
They are also available as \code{as.owin.*} functions to support
\code{\link{polyCub.midpoint}}.
}
\examples{
if (require("spatstat.geom") && require("sp")) {
    diamond <- list(x = c(1,2,1,0), y = c(1,2,3,2))  # anti-clockwise
    diamond.owin <- owin(poly = diamond)
    diamond.sp <- Polygon(lapply(diamond, rev))      # clockwise
    stopifnot(identical(xylist(diamond.sp), list(diamond)))
    diamond.owin_from_sp <- as.owin(diamond.sp)
    stopifnot(all.equal(diamond.owin, diamond.owin_from_sp))

    ## similarly works for Polygons and SpatialPolygons
    diamond.Ps <- as(diamond.sp, "Polygons")
    stopifnot(identical(diamond.owin, as.owin(diamond.Ps)))
    diamond.SpPs <- SpatialPolygons(list(diamond.Ps))
    stopifnot(identical(xylist(diamond.SpPs), list(diamond)))
    stopifnot(identical(diamond.owin, as.owin(diamond.SpPs)))
}
}
\seealso{
\code{\link{xylist}}
}
\author{
Sebastian Meyer
}
\keyword{methods}
\keyword{spatial}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotpolyf.R
\name{plotpolyf}
\alias{plotpolyf}
\title{Plot Polygonal Domain on Image of Bivariate Function}
\usage{
plotpolyf(polyregion, f, ..., npixel = 100, cuts = 15,
  col = rev(heat.colors(cuts + 1)), lwd = 3, xlim = NULL, ylim = NULL,
  use.lattice = TRUE, print.args = list())
}
\arguments{
\item{polyregion}{a polygonal domain.
The following classes are supported:
\code{"\link[spatstat.geom]{owin}"} from package \pkg{spatstat.geom},
\code{"gpc.poly"} from \pkg{gpclib},
\code{"\link[sp:SpatialPolygons-class]{SpatialPolygons}"},
\code{"\link[sp:Polygons-class]{Polygons}"},
and \code{"\link[sp:Polygon-class]{Polygon}"} from package \pkg{sp}, as well as
\code{"\link[sf:st_polygon]{(MULTI)POLYGON}"} from package \pkg{sf}.
(For these classes, \pkg{polyCub} knows how to get an \code{\link{xylist}}.)}

\item{f}{a two-dimensional real-valued function.
As its first argument it must take a coordinate matrix, i.e., a
numeric matrix with two columns, and it must return a numeric vector of
length the number of coordinates.}

\item{...}{further arguments for \code{f}.}

\item{npixel}{numeric vector of length 1 or 2 setting the number of pixels
in each dimension.}

\item{cuts}{number of cut points in the \eqn{z} dimension.
The range of function values will be divided into \code{cuts+1} levels.}

\item{col}{color vector used for the function levels.}

\item{lwd}{line width of the polygon edges.}

\item{xlim, ylim}{numeric vectors of length 2 setting the axis limits.
\code{NULL} means using the bounding box of \code{polyregion}.}

\item{use.lattice}{logical indicating if \pkg{lattice} graphics
(\code{\link[lattice]{levelplot}}) should be used.}

\item{print.args}{a list of arguments passed to \code{\link[lattice]{print.trellis}}
for plotting the produced \code{\link[lattice:trellis.object]{"trellis"}} object
(given \code{use.lattice = TRUE}). The latter will be returned without
explicit \code{print}ing if \code{print.args} is not a list.}
}
\description{
Produces a combined plot of a polygonal domain and an image of a bivariate
function, using either \code{\link[lattice:levelplot]{lattice::levelplot}}
or \code{\link{image}}.
}
\examples{
### a polygonal domain (a simplified version of spatstat.data::letterR$bdry)
letterR <- list(
    list(x = c(2.7, 3, 3.3, 3.9, 3.7, 3.4, 3.8, 3.7, 3.4, 2, 2, 2.7),
         y = c(1.7, 1.6, 0.7, 0.7, 1.3, 1.8, 2.2, 2.9, 3.3, 3.3, 0.7, 0.7)),
    list(x = c(2.6, 2.6, 3, 3.2, 3),
         y = c(2.2, 2.7, 2.7, 2.5, 2.2))
)

### f: isotropic exponential decay
fr <- function(r, rate = 1) dexp(r, rate = rate)
fcenter <- c(2,3)
f <- function (s, rate = 1) fr(sqrt(rowSums(t(t(s)-fcenter)^2)), rate = rate)

### plot
plotpolyf(letterR, f, use.lattice = FALSE)
plotpolyf(letterR, f, use.lattice = TRUE)
}
\author{
Sebastian Meyer
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/polyCub.midpoint.R
\name{polyCub.midpoint}
\alias{polyCub.midpoint}
\title{Two-Dimensional Midpoint Rule}
\usage{
polyCub.midpoint(polyregion, f, ..., eps = NULL, dimyx = NULL,
  plot = FALSE)
}
\arguments{
\item{polyregion}{a polygonal integration domain.
It can be any object coercible to the \pkg{spatstat.geom} class
\code{"\link[spatstat.geom]{owin}"} via a corresponding
\code{\link[spatstat.geom]{as.owin}}-method.
Note that this includes polygons of the classes \code{"gpc.poly"} and
\code{"\link[sp:SpatialPolygons-class]{SpatialPolygons}"}, because
\pkg{polyCub} defines methods \code{\link{as.owin.gpc.poly}} and
\code{\link{as.owin.SpatialPolygons}}, respectively.
\pkg{sf} also registers suitable \code{as.owin} methods for its
\code{"\link[sf:st_polygon]{(MULTI)POLYGON}"} classes.}

\item{f}{a two-dimensional real-valued function.
As its first argument it must take a coordinate matrix, i.e., a
numeric matrix with two columns, and it must return a numeric vector of
length the number of coordinates.}

\item{...}{further arguments for \code{f}.}

\item{eps}{width and height of the pixels (squares),
see \code{\link[spatstat.geom]{as.mask}}.}

\item{dimyx}{number of subdivisions in each dimension,
see \code{\link[spatstat.geom]{as.mask}}.}

\item{plot}{logical indicating if an illustrative plot of the numerical
integration should be produced.}
}
\value{
The approximated value of the integral of \code{f} over
\code{polyregion}.
}
\description{
The surface is converted to a binary pixel image
using the \code{\link[spatstat.geom]{as.im.function}} method from package
\CRANpkg{spatstat.geom} (Baddeley et al., 2015).
The integral under the surface is then approximated as the
sum over (pixel area * f(pixel midpoint)).
}
\examples{
## a function to integrate (here: isotropic zero-mean Gaussian density)
f <- function (s, sigma = 5)
    exp(-rowSums(s^2)/2/sigma^2) / (2*pi*sigma^2)

## a simple polygon as integration domain
hexagon <- list(
    list(x = c(7.33, 7.33, 3, -1.33, -1.33, 3),
         y = c(-0.5, 4.5, 7, 4.5, -0.5, -3))
)

if (require("spatstat.geom")) {
    hexagon.owin <- owin(poly = hexagon)

    show_midpoint <- function (eps)
    {
        plotpolyf(hexagon.owin, f, xlim = c(-8,8), ylim = c(-8,8),
                  use.lattice = FALSE)
        ## add evaluation points to plot
        with(as.mask(hexagon.owin, eps = eps),
             points(expand.grid(xcol, yrow), col = t(m), pch = 20))
        title(main = paste("2D midpoint rule with eps =", eps))
    }

    ## show nodes (eps = 0.5)
    show_midpoint(0.5)

    ## show pixel image (eps = 0.5)
    polyCub.midpoint(hexagon.owin, f, eps = 0.5, plot = TRUE)

    ## use a decreasing pixel size (increasing number of nodes)
    for (eps in c(5, 3, 1, 0.5, 0.3, 0.1))
        cat(sprintf("eps = \%.1f: \%.7f\n", eps,
                    polyCub.midpoint(hexagon.owin, f, eps = eps)))
}
}
\references{
Baddeley A, Rubak E, Turner R (2015).
\emph{Spatial Point Patterns: Methodology and Applications with R}.
Chapman and Hall/CRC Press, London.
}
\seealso{
Other polyCub-methods: 
\code{\link{polyCub}()},
\code{\link{polyCub.SV}()},
\code{\link{polyCub.exact.Gauss}()},
\code{\link{polyCub.iso}()}
}
\concept{polyCub-methods}
\keyword{math}
\keyword{spatial}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sfg2gpc.R
\name{sfg2gpc}
\alias{sfg2gpc}
\title{Convert polygonal \code{"sfg"} to \code{"gpc.poly"}}
\usage{
sfg2gpc(object)
}
\arguments{
\item{object}{a \code{"POLYGON"} or \code{"MULTIPOLYGON"} \code{"sfg"} object.}
}
\value{
The converted polygon of class \code{"gpc.poly"}.
If package \pkg{gpclib} is not available,
\code{sfg2gpc} will just return the \code{pts} slot of the
\code{"gpc.poly"} (no formal class) with a warning.
}
\description{
Package \pkg{polyCub} implements a converter from class
\code{"\link[sf:st_polygon]{(MULTI)POLYGON}"} of package \CRANpkg{sf}
to \code{"gpc.poly"} of package \CRANpkg{gpclib}
such that \code{\link{polyCub.exact.Gauss}}
can be used with simple feature polygons.
}
\note{
Package \pkg{gpclib} is required for the formal class
definition of a \code{"gpc.poly"}.
}
\examples{
\dontshow{if (requireNamespace("sf")) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
## use example polygons from
example(plotpolyf, ask = FALSE)
letterR  # a simple "xylist"

letterR.sfg <- sf::st_polygon(lapply(letterR, function(xy)
    rbind(cbind(xy$x, xy$y), c(xy$x[1], xy$y[1]))))
letterR.sfg
stopifnot(identical(letterR, xylist(letterR.sfg)))
\dontshow{
stopifnot(identical(rep(letterR, 2),
    xylist(sf::st_multipolygon(list(letterR.sfg, letterR.sfg)))))
}
## convert sf "POLYGON" to a "gpc.poly"
letterR.gpc_from_sfg <- sfg2gpc(letterR.sfg)
letterR.gpc_from_sfg
\dontshow{
if (is(letterR.gpc_from_sfg, "gpc.poly") && requireNamespace("spatstat.geom")) {
    letterR.xylist_from_gpc <- xylist(letterR.gpc_from_sfg) # with hole info
    stopifnot(identical(letterR, lapply(letterR.xylist_from_gpc, `[`, 1:2)))
}}
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link{xylist}}
}
\author{
Sebastian Meyer
}
\keyword{methods}
\keyword{spatial}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{isClosed}
\alias{isClosed}
\title{Check if Polygon is Closed}
\usage{
isClosed(coords)
}
\arguments{
\item{coords}{numeric coordinate matrix. It is interpreted by
\code{\link{xy.coords}}.}
}
\value{
logical
}
\description{
Check if the first and last coordinates of a coordinate matrix are
identical.
}
\keyword{internal}
\keyword{spatial}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{plot_polyregion}
\alias{plot_polyregion}
\title{Plots a Polygonal Domain (of Various Classes)}
\usage{
plot_polyregion(polyregion, lwd = 2, add = FALSE)
}
\arguments{
\item{polyregion}{a polygonal domain.
The following classes are supported:
\code{"\link[spatstat.geom]{owin}"} from package \pkg{spatstat.geom},
\code{"gpc.poly"} from \pkg{gpclib},
\code{"\link[sp:SpatialPolygons-class]{SpatialPolygons}"},
\code{"\link[sp:Polygons-class]{Polygons}"},
and \code{"\link[sp:Polygon-class]{Polygon}"} from package \pkg{sp}, as well as
\code{"\link[sf:st_polygon]{(MULTI)POLYGON}"} from package \pkg{sf}.
(For these classes, \pkg{polyCub} knows how to get an \code{\link{xylist}}.)}

\item{lwd}{line width of the polygon edges.}

\item{add}{logical. Add to existing plot?}
}
\description{
Plots a Polygonal Domain (of Various Classes)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{vecnorm}
\alias{vecnorm}
\title{Euclidean Vector Norm (Length)}
\usage{
vecnorm(x)
}
\arguments{
\item{x}{numeric vector.}
}
\value{
\code{sqrt(sum(x^2))}
}
\description{
This is nothing else than \code{sqrt(sum(x^2))}.
}
\keyword{internal}
\keyword{math}
